@@ -1,11 +1,15 @@
-/**
-  Copyright (c) 2011 Mellanox Technologies. All rights reserved.
-  $COPYRIGHT$
-
-  Additional copyrights may follow
-
-  $HEADER$
+/*
+ * Copyright (c) 2011      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2014      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
+ *
+ * $HEADER$
  */
+
 #include "coll_fca.h"
 
 /*
@@ -32,25 +36,6 @@ int mca_coll_fca_init_query(bool enable_progress_threads,
     return OMPI_SUCCESS;
 }
 
-static int have_remote_peers(ompi_group_t *group, size_t size, int *local_peers)
-{
-    ompi_proc_t *proc;
-    size_t i;
-    int ret;
-
-    *local_peers = 0;
-    ret = 0;
-    for (i = 0; i < size; ++i) {
-        proc = ompi_group_peer_lookup(group, i);
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
-            ++*local_peers;
-        } else {
-            ret = 1;
-        }
-    }
-    return ret;
-}
-
 static inline ompi_proc_t* __local_rank_lookup(ompi_communicator_t *comm, int rank)
 {
     return ompi_group_peer_lookup(comm->c_local_group, rank);
@@ -69,7 +54,7 @@ static int __get_local_ranks(mca_coll_fca_module_t *fca_module)
     fca_module->num_local_procs = 0;
     for (rank = 0; rank < ompi_comm_size(comm); ++rank) {
         proc = __local_rank_lookup(comm, rank);
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
             if (rank == fca_module->rank) {
                 fca_module->local_proc_idx = fca_module->num_local_procs;
             }
@@ -89,7 +74,7 @@ static int __get_local_ranks(mca_coll_fca_module_t *fca_module)
     i = 0;
     for (rank = 0; rank < ompi_comm_size(comm); ++rank) {
         proc = __local_rank_lookup(comm, rank);
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
             fca_module->local_ranks[i++] = rank;
         }
     }
@@ -104,8 +89,10 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
     ompi_communicator_t *comm = fca_module->comm;
     fca_comm_new_spec_t spec = {0,};
     int info_size, all_info_size;
-    void *all_info, *my_info;
-    int *rcounts, *displs;
+    void *all_info = NULL;
+    void *my_info = NULL;
+    int *rcounts = NULL;
+    int *displs = NULL;
     int i, rc, ret, comm_size = ompi_comm_size(fca_module->comm);
 
     /* call fca_get_rank_info() on node managers only*/
@@ -276,7 +263,7 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
     opal_list_t *c_cache;
     struct timeval start, end, seq_start, seq_end, par_start, par_end;
     int act_deep;
-    
+
 
     if(mca_coll_fca_component.fca_verbose == 10) {
         gettimeofday(&start, NULL);
@@ -292,7 +279,7 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
             int hash_index, part_of_hash_index;
 
             if(mca_coll_fca_component.fca_parallel_hash_calc == 1) {
-                
+
                 if(mca_coll_fca_component.fca_verbose == 10){
                     gettimeofday(&par_start, NULL);
                 }
@@ -358,9 +345,9 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
                 c_item = (mca_coll_fca_c_cache_item_t *)opal_list_get_next((opal_list_item_t *) c_item)){
             act_deep++;
             /* first check the size */
-            if( comm_size == c_item->size){
+            if( c_item && (comm_size == c_item->size)) {
                 /* then we have a potential cache hit */
-                ompi_comm_compare(comm, c_item->comm, &result); 
+                ompi_comm_compare(comm, c_item->comm, &result);
                 if( MPI_CONGRUENT == result) {
                     /* cache hit! Return the context and be done with it */
                     /* first bump the score */
@@ -373,12 +360,12 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
 
                     if(mca_coll_fca_component.fca_verbose == 10) {
                         gettimeofday(&end, NULL);
-                        
+
                         mca_coll_fca_component.fca_total_work_time =+
                             end.tv_sec - start.tv_sec + 1e-6 * (end.tv_usec - start.tv_usec);
-                        
+
                         mca_coll_fca_component.fca_cache_hit += 1;
-                        
+
                         if(act_deep>mca_coll_fca_component.fca_max_deep_in_cache)
                             mca_coll_fca_component.fca_max_deep_in_cache = act_deep;
                     }
@@ -421,7 +408,7 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
         c_item_new = OBJ_NEW(mca_coll_fca_c_cache_item_t);
         c_item_new->fca_comm_wrap = OBJ_NEW(mca_coll_fca_comm_wrap_t);
 
-        OBJ_RETAIN(comm); 
+        OBJ_RETAIN(comm);
 
         c_item_new->size = comm_size;
         c_item_new->comm = comm;
@@ -433,9 +420,9 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
     }
 
     if(mca_coll_fca_component.fca_verbose == 10) {
-        
+
         gettimeofday(&end, NULL);
-        
+
         mca_coll_fca_component.fca_total_work_time =+
             end.tv_sec - start.tv_sec + 1e-6 * (end.tv_usec - start.tv_usec);
 
@@ -529,11 +516,6 @@ static int mca_coll_fca_module_enable(mca_coll_base_module_t *module,
 }
 
 
-static int mca_coll_fca_ft_event(int state)
-{    
-    return OMPI_SUCCESS;
-}
-
 static void mca_coll_fca_module_clear(mca_coll_fca_module_t *fca_module)
 {
     fca_module->num_local_procs = 0;
@@ -567,7 +549,7 @@ static void mca_coll_fca_module_destruct(mca_coll_fca_module_t *fca_module)
 
         if (fca_module->fca_comm) {
             __destroy_fca_comm(fca_module);
-        } 
+        }
     }
 
     OBJ_RELEASE(fca_module->previous_barrier_module);
@@ -601,7 +583,7 @@ mca_coll_fca_comm_query(struct ompi_communicator_t *comm, int *priority)
 {
     mca_coll_base_module_t *module;
     int size = ompi_comm_size(comm);
-    int local_peers;
+    int local_peers = 0;
     mca_coll_fca_module_t *fca_module;
 
     *priority = 0;
@@ -613,7 +595,7 @@ mca_coll_fca_comm_query(struct ompi_communicator_t *comm, int *priority)
     if (size < mca_coll_fca_component.fca_np)
         goto exit;
 
-    if (!have_remote_peers(comm->c_local_group, size, &local_peers) || OMPI_COMM_IS_INTER(comm))
+    if (!ompi_group_have_remote_peers(comm->c_local_group) || OMPI_COMM_IS_INTER(comm))
         goto exit;
 
     fca_module = OBJ_NEW(mca_coll_fca_module_t);
@@ -621,7 +603,7 @@ mca_coll_fca_comm_query(struct ompi_communicator_t *comm, int *priority)
         goto exit;
 
     fca_module->super.coll_module_enable = mca_coll_fca_module_enable;
-    fca_module->super.ft_event        = mca_coll_fca_ft_event;
+    fca_module->super.ft_event        = NULL;
     fca_module->super.coll_allgather  = mca_coll_fca_component.fca_enable_allgather?  mca_coll_fca_allgather  : NULL;
     fca_module->super.coll_allgatherv = mca_coll_fca_component.fca_enable_allgatherv? mca_coll_fca_allgatherv : NULL;
     fca_module->super.coll_allreduce  = mca_coll_fca_component.fca_enable_allreduce?  mca_coll_fca_allreduce  : NULL;
@@ -698,7 +680,7 @@ static void mca_coll_fca_c_cache_item_destruct(mca_coll_fca_c_cache_item_t *item
         OBJ_RELEASE(item->fca_comm_wrap);
         /* OBJ_RELEASE(item->comm); */
     }
-} 
+}
 
 OBJ_CLASS_INSTANCE(mca_coll_fca_c_cache_item_t,
         opal_list_item_t,
