@@ -1,10 +1,13 @@
-/**
-  Copyright (c) 2011 Mellanox Technologies. All rights reserved.
-  $COPYRIGHT$
-
-  Additional copyrights may follow
-
-  $HEADER$
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
+/*
+ * Copyright (c) 2011 Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
+ *
+ * $HEADER$
  */
 #include "ompi_config.h"
 #include <stdio.h>
@@ -14,6 +17,7 @@
 
 #include "coll_hcoll.h"
 #include "opal/mca/installdirs/installdirs.h"
+#include "coll_hcoll_dtypes.h"
 
 /*
  * Public string showing the coll ompi_hcol component version number
@@ -31,34 +35,33 @@ mca_coll_hcoll_component_t mca_coll_hcoll_component = {
     /* First, the mca_component_t struct containing meta information
        about the component itfca */
     {
-        {
+        .collm_version = {
             MCA_COLL_BASE_VERSION_2_0_0,
 
             /* Component name and version */
-            "hcoll",
-            OMPI_MAJOR_VERSION,
-            OMPI_MINOR_VERSION,
-            OMPI_RELEASE_VERSION,
+            .mca_component_name = "hcoll",
+            MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                                  OMPI_RELEASE_VERSION),
 
             /* Component open and close functions */
-            hcoll_open,
-            hcoll_close,
-            NULL,
-            hcoll_register
+            .mca_open_component = hcoll_open,
+            .mca_close_component = hcoll_close,
+            .mca_register_component_params = hcoll_register,
         },
-        {
+        .collm_data = {
             /* The component is not checkpoint ready */
             MCA_BASE_METADATA_PARAM_NONE
         },
 
         /* Initialization / querying functions */
 
-        mca_coll_hcoll_init_query,
-        mca_coll_hcoll_comm_query,
+        .collm_init_query = mca_coll_hcoll_init_query,
+        .collm_comm_query = mca_coll_hcoll_comm_query,
     },
     90, /* priority */
     0,  /* verbose level */
-    1   /* hcoll_enable */
+    0,   /* hcoll_enable */
+    NULL /*hcoll version */
 };
 
 
@@ -189,30 +192,55 @@ static int hcoll_register(void)
 
     CHECK(reg_int("enable",NULL,
                   "[1|0|] Enable/Disable HCOL",
-                  1 /*enable by default*/,
+                  1,
                   &mca_coll_hcoll_component.hcoll_enable,
                   0));
 
     CHECK(reg_int("np",NULL,
                   "Minimal number of processes in the communicator"
                   " for the corresponding hcoll context to be created (default: 32)",
-                  2 /*enable by default*/,
+                  2,
                   &mca_coll_hcoll_component.hcoll_np,
                   0));
 
     CHECK(reg_int("datatype_fallback",NULL,
                   "[1|0|] Enable/Disable user defined dattypes fallback",
-                  1 /*enable by default*/,
+                  1,
                   &mca_coll_hcoll_component.hcoll_datatype_fallback,
                   0));
-
+#if HCOLL_API >= HCOLL_VERSION(3,6)
+    CHECK(reg_int("dts",NULL,
+                  "[1|0|] Enable/Disable derived types support",
+                  1,
+                  &mca_coll_hcoll_component.derived_types_support_enabled,
+                  0));
+#else
+    mca_coll_hcoll_component.derived_types_support_enabled = 0;
+#endif
+    mca_coll_hcoll_component.compiletime_version = HCOLL_VERNO_STRING;
+    mca_base_component_var_register(&mca_coll_hcoll_component.super.collm_version,
+            MCA_COMPILETIME_VER,
+            "Version of the libhcoll library with which Open MPI was compiled",
+            MCA_BASE_VAR_TYPE_VERSION_STRING,
+            NULL, 0, 0,
+            OPAL_INFO_LVL_3,
+            MCA_BASE_VAR_SCOPE_READONLY,
+            &mca_coll_hcoll_component.compiletime_version);
+    mca_coll_hcoll_component.runtime_version = hcoll_get_version();
+    mca_base_component_var_register(&mca_coll_hcoll_component.super.collm_version,
+            MCA_RUNTIME_VER,
+            "Version of the libhcoll library with which Open MPI is running",
+            MCA_BASE_VAR_TYPE_VERSION_STRING,
+            NULL, 0, 0,
+            OPAL_INFO_LVL_3,
+            MCA_BASE_VAR_SCOPE_READONLY,
+            &mca_coll_hcoll_component.runtime_version);
 
     return ret;
 }
 
 static int hcoll_open(void)
 {
-    int rc;
     mca_coll_hcoll_component_t *cm;
     cm  = &mca_coll_hcoll_component;
 
@@ -222,6 +250,20 @@ static int hcoll_open(void)
     hcoll_rte_fns_setup();
 
     cm->libhcoll_initialized = false;
+
+    /* Register memory hooks */
+    if ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT) ==
+        ((OPAL_MEMORY_FREE_SUPPORT | OPAL_MEMORY_MUNMAP_SUPPORT) &
+         opal_mem_hooks_support_level()))
+    {
+        setenv("MXM_HCOLL_MEM_ON_DEMAND_MAP", "y", 0);
+        HCOL_VERBOSE(1, "Enabling on-demand memory mapping");
+        cm->using_mem_hooks = 1;
+    } else {
+        HCOL_VERBOSE(1, "Disabling on-demand memory mapping");
+        cm->using_mem_hooks = 0;
+    }
+
     return OMPI_SUCCESS;
 }
 
@@ -235,9 +277,17 @@ static int hcoll_close(void)
         return OMPI_SUCCESS;
     }
 
+    if (cm->using_mem_hooks) {
+        opal_mem_hooks_unregister_release(mca_coll_hcoll_mem_release_cb);
+    }
+
+#if HCOLL_API >= HCOLL_VERSION(3,2)
+    hcoll_free_init_opts(cm->init_opts);
+#endif
+
     HCOL_VERBOSE(5,"HCOLL FINALIZE");
     rc = hcoll_finalize();
-
+    OBJ_DESTRUCT(&cm->dtypes);
     opal_progress_unregister(mca_coll_hcoll_progress);
     if (HCOLL_SUCCESS != rc){
         HCOL_VERBOSE(1,"Hcol library finalize failed");
