@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,20 +6,22 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "ompi_config.h"
 
-#include "opal/mca/mca.h"
+#include "ompi/mca/mca.h"
 #include "opal/util/output.h"
 #include "opal/mca/base/base.h"
 
@@ -46,12 +49,14 @@ mca_mtl_base_module_t *ompi_mtl = NULL;
  * need to reexamine this at a later time.
  */
 int
-ompi_mtl_base_select(bool enable_progress_threads,
-                     bool enable_mpi_threads)
+ompi_mtl_base_select (bool enable_progress_threads,
+                      bool enable_mpi_threads,
+                      int *priority)
 {
     int ret = OMPI_ERR_NOT_FOUND;
     mca_mtl_base_component_t *best_component = NULL;
     mca_mtl_base_module_t *best_module = NULL;
+    int best_priority;
 
     /*
      * Select the best component
@@ -59,12 +64,13 @@ ompi_mtl_base_select(bool enable_progress_threads,
     if( OPAL_SUCCESS != mca_base_select("mtl", ompi_mtl_base_framework.framework_output,
                                         &ompi_mtl_base_framework.framework_components,
                                         (mca_base_module_t **) &best_module,
-                                        (mca_base_component_t **) &best_component) ) {
+                                        (mca_base_component_t **) &best_component,
+                                        &best_priority) ) {
         /* notify caller that no available component found */
         return ret;
     }
 
-    opal_output_verbose( 10, ompi_mtl_base_framework.framework_output, 
+    opal_output_verbose( 10, ompi_mtl_base_framework.framework_output,
                          "select: initializing %s component %s",
                          best_component->mtl_version.mca_type_name,
                          best_component->mtl_version.mca_component_name );
@@ -79,15 +85,16 @@ ompi_mtl_base_select(bool enable_progress_threads,
                              "select: init returned success");
         ompi_mtl_base_selected_component = best_component;
         ompi_mtl = best_module;
+        *priority = best_priority;
         ret = OMPI_SUCCESS;
     }
 
     /* All done */
     if (NULL == ompi_mtl) {
-        opal_output_verbose( 10, ompi_mtl_base_framework.framework_output, 
+        opal_output_verbose( 10, ompi_mtl_base_framework.framework_output,
                              "select: no component selected");
     } else {
-        opal_output_verbose( 10, ompi_mtl_base_framework.framework_output, 
+        opal_output_verbose( 10, ompi_mtl_base_framework.framework_output,
                              "select: component %s selected",
                              ompi_mtl_base_selected_component->
                              mtl_version.mca_component_name );
