@@ -1,5 +1,8 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (C) Mellanox Technologies Ltd. 2001-2011.  ALL RIGHTS RESERVED.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,15 +21,22 @@
 #ifndef MXM_VERSION
 #define MXM_VERSION(major, minor) (((major)<<MXM_MAJOR_BIT)|((minor)<<MXM_MINOR_BIT))
 #endif
-#if MXM_API < MXM_VERSION(2, 0)
+
+#if MXM_API < MXM_VERSION(1,5)
+#error "Unsupported MXM version, version 1.5 or above required"
+#endif
+
+#if MXM_API < MXM_VERSION(2,0)
 #include <mxm/api/mxm_addr.h>
 #endif
 
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/mtl/mtl.h"
 #include "ompi/mca/mtl/base/base.h"
-#include "ompi/class/ompi_free_list.h"
+#include "opal/class/opal_free_list.h"
 
+#include "opal/util/output.h"
+#include "opal/util/show_help.h"
 #include "opal/datatype/opal_convertor.h"
 
 #include "mtl_mxm_debug.h"
@@ -36,7 +46,8 @@ BEGIN_C_DECLS
 /* MTL interface functions */
 extern int ompi_mtl_mxm_add_procs(struct mca_mtl_base_module_t* mtl,
                                   size_t nprocs, struct ompi_proc_t** procs);
-
+extern int ompi_mtl_add_single_proc(struct mca_mtl_base_module_t *mtl,
+                                  struct ompi_proc_t* procs);
 extern int ompi_mtl_mxm_del_procs(struct mca_mtl_base_module_t* mtl,
                                   size_t nprocs, struct ompi_proc_t** procs);
 
@@ -87,9 +98,8 @@ extern int ompi_mtl_mxm_finalize(struct mca_mtl_base_module_t* mtl);
 
 int ompi_mtl_mxm_module_init(void);
 
-#if MXM_API >= MXM_VERSION(1,5)
 struct ompi_mtl_mxm_message_t {
-    ompi_free_list_item_t super;
+    opal_free_list_item_t super;
 
     mxm_mq_h mq;
     mxm_conn_h conn;
@@ -100,7 +110,6 @@ struct ompi_mtl_mxm_message_t {
 };
 typedef struct ompi_mtl_mxm_message_t ompi_mtl_mxm_message_t;
 OBJ_CLASS_DECLARATION(ompi_mtl_mxm_message_t);
-#endif
 
 END_C_DECLS
 
