@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,6 +11,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2010-2012 Sandia National Laboratories.  All rights reserved.
+ * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,18 +25,22 @@
 
 #include "opal/mca/event/event.h"
 #include "opal/util/output.h"
-#include "ompi/runtime/ompi_module_exchange.h"
+#include "opal/mca/pmix/pmix.h"
 
 #include "mtl_portals4.h"
 #include "mtl_portals4_request.h"
 #include "mtl_portals4_recv_short.h"
 #include "mtl_portals4_message.h"
+#include "ompi/runtime/mpiruntime.h"
+
+static int param_priority;
 
 static int ompi_mtl_portals4_component_register(void);
 static int ompi_mtl_portals4_component_open(void);
 static int ompi_mtl_portals4_component_close(void);
-static mca_mtl_base_module_t* 
-ompi_mtl_portals4_component_init(bool enable_progress_threads, 
+static int ompi_mtl_portals4_component_query(mca_base_module_t **module, int *priority);
+static mca_mtl_base_module_t*
+ompi_mtl_portals4_component_init(bool enable_progress_threads,
                                  bool enable_mpi_threads);
 
 OMPI_MODULE_DECLSPEC extern mca_mtl_base_component_2_0_0_t mca_mtl_portals4_component;
@@ -42,24 +50,23 @@ mca_mtl_base_component_2_0_0_t mca_mtl_portals4_component = {
     /* First, the mca_base_component_t struct containing meta
      * information about the component itself */
 
-    {
+    .mtl_version = {
         MCA_MTL_BASE_VERSION_2_0_0,
 
-        "portals4", /* MCA component name */
-        OMPI_MAJOR_VERSION,  /* MCA component major version */
-        OMPI_MINOR_VERSION,  /* MCA component minor version */
-        OMPI_RELEASE_VERSION,  /* MCA component release version */
-        ompi_mtl_portals4_component_open,  /* component open */
-        ompi_mtl_portals4_component_close, /* component close */
-        NULL,
-        ompi_mtl_portals4_component_register
+        .mca_component_name = "portals4",
+        MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                              OMPI_RELEASE_VERSION),
+        .mca_open_component = ompi_mtl_portals4_component_open,
+        .mca_close_component = ompi_mtl_portals4_component_close,
+        .mca_query_component = ompi_mtl_portals4_component_query,
+        .mca_register_component_params = ompi_mtl_portals4_component_register,
     },
-    {
+    .mtl_data = {
         /* The component is not checkpoint ready */
         MCA_BASE_METADATA_PARAM_NONE
     },
 
-    ompi_mtl_portals4_component_init,  /* component init */
+    .mtl_init = ompi_mtl_portals4_component_init,
 };
 
 static mca_base_var_enum_value_t long_protocol_values[] = {
@@ -68,12 +75,48 @@ static mca_base_var_enum_value_t long_protocol_values[] = {
     {0, NULL}
 };
 
+OBJ_CLASS_INSTANCE(ompi_mtl_portals4_rndv_get_frag_t,
+                   opal_free_list_item_t,
+                   NULL, NULL);
+
 static int
 ompi_mtl_portals4_component_register(void)
 {
     mca_base_var_enum_t *new_enum;
     int ret;
 
+    ompi_mtl_portals4.use_logical = 0;
+    (void) mca_base_component_var_register(&mca_mtl_portals4_component.mtl_version,
+                                           "use_logical",
+                                           "Use the logical to physical table to accelerate portals4 adressing: 1 (true) : 0 (false)",
+                                           MCA_BASE_VAR_TYPE_INT,
+                                           NULL,
+                                           0,
+                                           0,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_mtl_portals4.use_logical);
+
+    param_priority = 10;
+    (void) mca_base_component_var_register (&mca_mtl_portals4_component.mtl_version,
+                                            "priority", "Priority of the Portals4 MTL component",
+                                            MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
+                                            OPAL_INFO_LVL_9,
+                                            MCA_BASE_VAR_SCOPE_READONLY,
+                                            &param_priority);
+    ompi_mtl_portals4.short_limit = 2 * 1024;
+    (void) mca_base_component_var_register(&mca_mtl_portals4_component.mtl_version,
+                                           "short_limit",
+                                           "Size limit for short messages",
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_LONG_LONG,
+                                           NULL,
+                                           0,
+                                           0,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_mtl_portals4.short_limit);
+
+
     ompi_mtl_portals4.eager_limit = 2 * 1024;
     (void) mca_base_component_var_register(&mca_mtl_portals4_component.mtl_version,
                                            "eager_limit",
@@ -146,6 +189,19 @@ ompi_mtl_portals4_component_register(void)
                                            OPAL_INFO_LVL_5,
                                            MCA_BASE_VAR_SCOPE_READONLY,
                                            &ompi_mtl_portals4.protocol);
+
+    ompi_mtl_portals4.max_msg_size_mtl = PTL_SIZE_MAX;
+    (void) mca_base_component_var_register(&mca_mtl_portals4_component.mtl_version,
+                                           "max_msg_size",
+                                           "Max size supported by portals4 (above that, a message is cut into messages less than that size)",
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_LONG,
+                                           NULL,
+                                           0,
+                                           0,
+                                           OPAL_INFO_LVL_5,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &ompi_mtl_portals4.max_msg_size_mtl);
+
     OBJ_RELEASE(new_enum);
     if (0 > ret) {
         return OMPI_ERR_NOT_SUPPORTED;
@@ -157,7 +213,7 @@ ompi_mtl_portals4_component_register(void)
 static int
 ompi_mtl_portals4_component_open(void)
 {
-    ompi_mtl_portals4.base.mtl_request_size = 
+    ompi_mtl_portals4.base.mtl_request_size =
         sizeof(ompi_mtl_portals4_request_t) -
         sizeof(struct mca_mtl_request_t);
 
@@ -170,43 +226,80 @@ ompi_mtl_portals4_component_open(void)
 #endif
                         );
     opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                        "Eager limit: %d", (int) 
+                        "Max message size: %lu", (unsigned long)
+                        ompi_mtl_portals4.max_msg_size_mtl);
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                        "Short limit: %d", (int)
+                        ompi_mtl_portals4.short_limit);
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                        "Eager limit: %d", (int)
                         ompi_mtl_portals4.eager_limit);
-    opal_output_verbose(1, ompi_mtl_base_framework.framework_output, 
-                        "Short receive blocks: %d", 
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                        "Short receive blocks: %d",
                         ompi_mtl_portals4.recv_short_num);
-    opal_output_verbose(1, ompi_mtl_base_framework.framework_output, 
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
                         "Send queue size: %d", ompi_mtl_portals4.send_queue_size);
-    opal_output_verbose(1, ompi_mtl_base_framework.framework_output, 
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
                         "Recv queue size: %d", ompi_mtl_portals4.recv_queue_size);
-    opal_output_verbose(1, ompi_mtl_base_framework.framework_output, 
-                        "Long protocol: %s", 
+    opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                        "Long protocol: %s",
                         (ompi_mtl_portals4.protocol == eager) ? "Eager" :
                         (ompi_mtl_portals4.protocol == rndv) ? "Rendezvous" :
                          "Other");
 
     OBJ_CONSTRUCT(&ompi_mtl_portals4.fl_message, opal_free_list_t);
     opal_free_list_init(&ompi_mtl_portals4.fl_message,
-                        sizeof(ompi_mtl_portals4_message_t) + 
+                        sizeof(ompi_mtl_portals4_message_t) +
                         ompi_mtl_portals4.eager_limit,
+                        opal_cache_line_size,
                         OBJ_CLASS(ompi_mtl_portals4_message_t),
-                        1, -1, 1);
+                        0, 0, 1, -1, 1, NULL, 0, NULL, NULL, NULL);
+
+    OBJ_CONSTRUCT(&ompi_mtl_portals4.fl_rndv_get_frag, opal_free_list_t);
+    opal_free_list_init(&ompi_mtl_portals4.fl_rndv_get_frag,
+                        sizeof(ompi_mtl_portals4_rndv_get_frag_t),
+                        opal_cache_line_size,
+                        OBJ_CLASS(ompi_mtl_portals4_rndv_get_frag_t),
+                        0, 0, 1, -1, 1, NULL, 0, NULL, NULL, NULL);
 
     ompi_mtl_portals4.ni_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.send_eq_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.recv_eq_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.zero_md_h = PTL_INVALID_HANDLE;
-
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    ompi_mtl_portals4.send_md_hs = NULL;
-#else
     ompi_mtl_portals4.send_md_h = PTL_INVALID_HANDLE;
-#endif
-
     ompi_mtl_portals4.long_overflow_me_h = PTL_INVALID_HANDLE;
     ompi_mtl_portals4.recv_idx = (ptl_pt_index_t) ~0UL;
     ompi_mtl_portals4.read_idx = (ptl_pt_index_t) ~0UL;
 
+    ompi_mtl_portals4.need_init=1;
+
+#if OMPI_MTL_PORTALS4_FLOW_CONTROL
+    ompi_mtl_portals4.use_flowctl=1;
+#else
+    ompi_mtl_portals4.use_flowctl=0;
+#endif
+
+    return OMPI_SUCCESS;
+}
+
+#define NEED_ALL_PROCS (ompi_mtl_portals4.use_logical || ompi_mtl_portals4.use_flowctl)
+
+static int
+ompi_mtl_portals4_component_query(mca_base_module_t **module, int *priority)
+{
+    /*
+     * assume if portals4 MTL was compiled, the user wants it
+     */
+
+    *priority = param_priority;
+    *module = (mca_base_module_t *)&ompi_mtl_portals4.base;
+
+    if (NEED_ALL_PROCS) {
+        /* let the pml know we need add_procs to be calls with all the
+         * procs in the job */
+        ompi_mtl_portals4.base.mtl_flags |= MCA_MTL_BASE_FLAG_REQUIRE_WORLD;
+    }
+
     return OMPI_SUCCESS;
 }
 
@@ -226,8 +319,13 @@ ompi_mtl_portals4_component_init(bool enable_progress_threads,
 {
     int ret;
     ptl_process_t id;
-    ptl_md_t md;
-    ptl_me_t me;
+    ptl_ni_limits_t actual_limits;
+
+    if (enable_mpi_threads && ompi_mpi_thread_multiple) {
+        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
+                            "mtl portals4 is initialized for threads");
+    }
+
 
     /* Initialize Portals and create a physical, matching interface */
     ret = PtlInit();
@@ -238,11 +336,18 @@ ompi_mtl_portals4_component_init(bool enable_progress_threads,
         return NULL;
     }
 
-    ret = PtlNIInit(PTL_IFACE_DEFAULT,
+    if (ompi_mtl_portals4.use_logical)
+        ret = PtlNIInit(PTL_IFACE_DEFAULT,
+                    PTL_NI_LOGICAL | PTL_NI_MATCHING,
+                    PTL_PID_ANY,
+                    NULL,
+                    &actual_limits,
+                    &ompi_mtl_portals4.ni_h);
+    else ret = PtlNIInit(PTL_IFACE_DEFAULT,
                     PTL_NI_PHYSICAL | PTL_NI_MATCHING,
                     PTL_PID_ANY,
                     NULL,
-                    NULL,
+                    &actual_limits,
                     &ompi_mtl_portals4.ni_h);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
@@ -251,6 +356,30 @@ ompi_mtl_portals4_component_init(bool enable_progress_threads,
         goto error;
     }
 
+    if (actual_limits.max_msg_size < ompi_mtl_portals4.max_msg_size_mtl)
+        ompi_mtl_portals4.max_msg_size_mtl = actual_limits.max_msg_size;
+    OPAL_OUTPUT_VERBOSE((10, ompi_mtl_base_framework.framework_output,
+        "Due to portals4 and user configuration messages will not go over the size of %lu", ompi_mtl_portals4.max_msg_size_mtl));
+
+    if (ompi_comm_rank(MPI_COMM_WORLD) == 0) {
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_entries=%d", actual_limits.max_entries);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_unexpected_headers=%d", actual_limits.max_unexpected_headers);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_mds=%d", actual_limits.max_mds);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_eqs=%d", actual_limits.max_eqs);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_cts=%d", actual_limits.max_cts);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_pt_index=%d", actual_limits.max_pt_index);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_iovecs=%d", actual_limits.max_iovecs);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_list_size=%d", actual_limits.max_list_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_triggered_ops=%d", actual_limits.max_triggered_ops);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_msg_size=%ld", actual_limits.max_msg_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_atomic_size=%ld", actual_limits.max_atomic_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_fetch_atomic_size=%ld", actual_limits.max_fetch_atomic_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_waw_ordered_size=%ld", actual_limits.max_waw_ordered_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_war_ordered_size=%ld", actual_limits.max_war_ordered_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "max_volatile_size=%ld", actual_limits.max_volatile_size);
+        opal_output_verbose(10, ompi_mtl_base_framework.framework_output, "features=%u", actual_limits.features);
+    }
+
     ret = PtlGetUid(ompi_mtl_portals4.ni_h, &ompi_mtl_portals4.uid);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
@@ -260,16 +389,21 @@ ompi_mtl_portals4_component_init(bool enable_progress_threads,
     }
 
     /* Publish our NID/PID in the modex */
-    ret = PtlGetId(ompi_mtl_portals4.ni_h, &id);
+    ret = PtlGetPhysId(ompi_mtl_portals4.ni_h, &id);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlGetId failed: %d\n",
+                            "%s:%d: PtlGetPhysId failed: %d\n",
                             __FILE__, __LINE__, ret);
         goto error;
     }
 
-    ret = ompi_modex_send(&mca_mtl_portals4_component.mtl_version,
-                          &id, sizeof(id));
+    ompi_mtl_portals4.ptl_process_id = id;
+    OPAL_OUTPUT_VERBOSE((90, ompi_mtl_base_framework.framework_output,
+        "PtlGetPhysId rank=%x nid=%x pid=%x\n", id.rank, id.phys.nid, id.phys.pid));
+
+    OPAL_MODEX_SEND(ret, OPAL_PMIX_GLOBAL,
+                    &mca_mtl_portals4_component.mtl_version,
+                    &id, sizeof(id));
     if (OMPI_SUCCESS != ret) {
         opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
                             "%s:%d: ompi_modex_send failed: %d\n",
@@ -281,233 +415,10 @@ ompi_mtl_portals4_component_init(bool enable_progress_threads,
                          "My nid,pid = %x,%x",
                          id.phys.nid, id.phys.pid));
 
-    /* create event queues */
-    ret = PtlEQAlloc(ompi_mtl_portals4.ni_h,
-                     ompi_mtl_portals4.send_queue_size,
-                     &ompi_mtl_portals4.send_eq_h);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlEQAlloc failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-    ret = PtlEQAlloc(ompi_mtl_portals4.ni_h,
-                     ompi_mtl_portals4.recv_queue_size,
-                     &ompi_mtl_portals4.recv_eq_h);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlEQAlloc failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    /* Create send and long message (read) portal table entries */
-    ret = PtlPTAlloc(ompi_mtl_portals4.ni_h,
-                     PTL_PT_ONLY_USE_ONCE | 
-                     PTL_PT_ONLY_TRUNCATE | 
-                     PTL_PT_FLOWCTRL,
-                     ompi_mtl_portals4.recv_eq_h,
-                     REQ_RECV_TABLE_ID,
-                     &ompi_mtl_portals4.recv_idx);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlPTAlloc failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-    ret = PtlPTAlloc(ompi_mtl_portals4.ni_h,
-                     PTL_PT_ONLY_USE_ONCE |
-                     PTL_PT_ONLY_TRUNCATE,
-                     ompi_mtl_portals4.send_eq_h,
-                     REQ_READ_TABLE_ID,
-                     &ompi_mtl_portals4.read_idx);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlPTAlloc failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    /* bind zero-length md for sending acks */
-    md.start     = NULL;
-    md.length    = 0;
-    md.options   = 0;
-    md.eq_handle = PTL_EQ_NONE;
-    md.ct_handle = PTL_CT_NONE;
-
-    ret = PtlMDBind(ompi_mtl_portals4.ni_h,
-                    &md,
-                    &ompi_mtl_portals4.zero_md_h); 
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlMDBind failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    /* Bind MD/MDs across all memory.  We prefer (for obvious reasons)
-       to have a single MD across all of memory */
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    {
-        int i;
-        int num_mds = ompi_mtl_portals4_get_num_mds();
-        ptl_size_t size = (1ULL << OMPI_PORTALS4_MAX_MD_SIZE) - 1;
-        ptl_size_t offset_unit = (1ULL << OMPI_PORTALS4_MAX_MD_SIZE) / 2;
-
-        ompi_mtl_portals4.send_md_hs = malloc(sizeof(ptl_handle_md_t) * num_mds);
-        if (NULL == ompi_mtl_portals4.send_md_hs) {
-            opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                                "%s:%d: Error allocating MD array",
-                                __FILE__, __LINE__);
-            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-            goto error;
-        }
-
-        for (i = 0 ; i < num_mds ; ++i) {
-            ompi_mtl_portals4.send_md_hs[i] = PTL_INVALID_HANDLE;
-        }
-
-        for (i = 0 ; i < num_mds ; ++i) {
-            md.start = (char*) (offset_unit * i);
-            md.length = (i - 1 == num_mds) ? size / 2 : size;
-            md.options = 0;
-            md.eq_handle = ompi_mtl_portals4.send_eq_h;
-            md.ct_handle = PTL_CT_NONE;
-
-            opal_output_verbose(50, ompi_mtl_base_framework.framework_output,
-                                "Binding md from %p of length %lx",
-                                md.start, md.length);
-
-            ret = PtlMDBind(ompi_mtl_portals4.ni_h,
-                            &md,
-                            &ompi_mtl_portals4.send_md_hs[i]); 
-            if (PTL_OK != ret) {
-                opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                                    "%s:%d: PtlMDBind failed: %d\n",
-                                    __FILE__, __LINE__, ret);
-                goto error;
-            }
-        }
-    }
-#else
-        md.start = 0;
-        md.length = PTL_SIZE_MAX;
-        md.options = 0;
-        md.eq_handle = ompi_mtl_portals4.send_eq_h;
-        md.ct_handle = PTL_CT_NONE;
-
-        ret = PtlMDBind(ompi_mtl_portals4.ni_h,
-                        &md,
-                        &ompi_mtl_portals4.send_md_h); 
-        if (PTL_OK != ret) {
-            opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                                "%s:%d: PtlMDBind failed: %d\n",
-                                __FILE__, __LINE__, ret);
-            goto error;
-        }
-#endif
-
-    /* Handle long overflows */
-    me.start = NULL;
-    me.length = 0;
-    me.ct_handle = PTL_CT_NONE;
-    me.min_free = 0;
-    me.uid = ompi_mtl_portals4.uid;
-    me.options = PTL_ME_OP_PUT | 
-        PTL_ME_EVENT_LINK_DISABLE |
-        PTL_ME_EVENT_COMM_DISABLE | 
-        PTL_ME_EVENT_UNLINK_DISABLE;
-    me.match_id.phys.nid = PTL_NID_ANY;
-    me.match_id.phys.pid = PTL_PID_ANY;
-    me.match_bits = MTL_PORTALS4_LONG_MSG;
-    me.ignore_bits = MTL_PORTALS4_CONTEXT_MASK | 
-        MTL_PORTALS4_SOURCE_MASK | 
-        MTL_PORTALS4_TAG_MASK;
-    ret = PtlMEAppend(ompi_mtl_portals4.ni_h,
-                      ompi_mtl_portals4.recv_idx,
-                      &me,
-                      PTL_OVERFLOW_LIST,
-                      NULL,
-                      &ompi_mtl_portals4.long_overflow_me_h);
-    if (PTL_OK != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: PtlMEAppend failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    /* attach short unex recv blocks */
-    ret = ompi_mtl_portals4_recv_short_init();
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: short receive block initialization failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
-    ompi_mtl_portals4.opcount = 0;
-#if OPAL_ENABLE_DEBUG
-    ompi_mtl_portals4.recv_opcount = 0;
-#endif
-
-#if OMPI_MTL_PORTALS4_FLOW_CONTROL
-    ret = ompi_mtl_portals4_flowctl_init();
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: ompi_mtl_portals4_flowctl_init failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-#endif
-
-    /* activate progress callback */
-    ret = opal_progress_register(ompi_mtl_portals4_progress);
-    if (OMPI_SUCCESS != ret) {
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: opal_progress_register failed: %d\n",
-                            __FILE__, __LINE__, ret);
-        goto error;
-    }
-
+    ompi_mtl_portals4.base.mtl_max_tag = MTL_PORTALS4_MAX_TAG;
     return &ompi_mtl_portals4.base;
 
  error:
-    if (!PtlHandleIsEqual(ompi_mtl_portals4.long_overflow_me_h, PTL_INVALID_HANDLE)) {
-        PtlMEUnlink(ompi_mtl_portals4.long_overflow_me_h);
-    }
-    if (!PtlHandleIsEqual(ompi_mtl_portals4.zero_md_h, PTL_INVALID_HANDLE)) {
-        PtlMDRelease(ompi_mtl_portals4.zero_md_h);
-    }
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    if (NULL != ompi_mtl_portals4.send_md_hs) {
-        int i;
-        int num_mds = ompi_mtl_portals4_get_num_mds();
-
-        for (i = 0 ; i < num_mds ; ++i) {
-            if (!PtlHandleIsEqual(ompi_mtl_portals4.send_md_hs[i], PTL_INVALID_HANDLE)) {
-                PtlMDRelease(ompi_mtl_portals4.send_md_hs[i]);
-            }
-        }
-
-        free(ompi_mtl_portals4.send_md_hs);
-    }
-#else
-    if (!PtlHandleIsEqual(ompi_mtl_portals4.send_md_h, PTL_INVALID_HANDLE)) {
-        PtlMDRelease(ompi_mtl_portals4.send_md_h);
-    }
-#endif
-    if (ompi_mtl_portals4.read_idx != (ptl_pt_index_t) ~0UL) {
-        PtlPTFree(ompi_mtl_portals4.ni_h, ompi_mtl_portals4.read_idx);
-    }
-    if (ompi_mtl_portals4.recv_idx != (ptl_pt_index_t) ~0UL) {
-        PtlPTFree(ompi_mtl_portals4.ni_h, ompi_mtl_portals4.recv_idx);
-    }
-    if (!PtlHandleIsEqual(ompi_mtl_portals4.send_eq_h, PTL_INVALID_HANDLE)) {
-        PtlEQFree(ompi_mtl_portals4.send_eq_h);
-    }
-    if (!PtlHandleIsEqual(ompi_mtl_portals4.recv_eq_h, PTL_INVALID_HANDLE)) {
-        PtlEQFree(ompi_mtl_portals4.recv_eq_h);
-    }
     return NULL;
 }
 
@@ -578,6 +489,7 @@ ompi_mtl_portals4_progress(void)
     unsigned int which;
     ptl_event_t ev;
     ompi_mtl_portals4_base_request_t *ptl_request;
+    ompi_mtl_portals4_rndv_get_frag_t *rndv_get_frag;
 
     while (true) {
 	ret = PtlEQPoll(ompi_mtl_portals4.eqs_h, 2, 0, &ev, &which);
@@ -589,7 +501,6 @@ ompi_mtl_portals4_progress(void)
             case PTL_EVENT_GET:
             case PTL_EVENT_PUT:
             case PTL_EVENT_PUT_OVERFLOW:
-            case PTL_EVENT_REPLY:
             case PTL_EVENT_SEND:
             case PTL_EVENT_ACK:
             case PTL_EVENT_AUTO_FREE:
@@ -607,6 +518,18 @@ ompi_mtl_portals4_progress(void)
                 }
                 break;
 
+            case PTL_EVENT_REPLY:
+                if (NULL != ev.user_ptr) {
+                    rndv_get_frag = ev.user_ptr;
+                    ret = rndv_get_frag->event_callback(&ev, rndv_get_frag);
+                    if (OMPI_SUCCESS != ret) {
+                        opal_output(ompi_mtl_base_framework.framework_output,
+                                    "Error returned from target event callback: %d", ret);
+                        abort();
+                    }
+                }
+                break;
+
             case PTL_EVENT_PT_DISABLED:
 #if OMPI_MTL_PORTALS4_FLOW_CONTROL
                 OPAL_OUTPUT_VERBOSE((10, ompi_mtl_base_framework.framework_output,
@@ -649,7 +572,7 @@ ompi_mtl_portals4_progress(void)
     }
 
 #if OMPI_MTL_PORTALS4_FLOW_CONTROL
-    if (OPAL_UNLIKELY(0 == count && 
+    if (OPAL_UNLIKELY(0 == count &&
                       0 != opal_list_get_size(&ompi_mtl_portals4.flowctl.pending_sends))) {
         ompi_mtl_portals4_pending_list_progress();
     }
@@ -657,3 +580,4 @@ ompi_mtl_portals4_progress(void)
 
     return count;
 }
+
