@@ -1,11 +1,14 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2011-2013 Sandia National Laboratories.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -15,7 +18,6 @@
 #include "ompi/mca/osc/base/base.h"
 #include "ompi/mca/osc/base/osc_base_obj_convert.h"
 #include "ompi/request/request.h"
-#include "ompi/class/ompi_free_list.h"
 
 #include "osc_portals4.h"
 #include "osc_portals4_request.h"
@@ -34,25 +36,22 @@ static int component_select(struct ompi_win_t *win, void **base, size_t size, in
 
 ompi_osc_portals4_component_t mca_osc_portals4_component = {
     { /* ompi_osc_base_component_t */
-        { /* ompi_base_component_t */
+        .osc_version = {
             OMPI_OSC_BASE_VERSION_3_0_0,
-            "portals4",
-            OMPI_MAJOR_VERSION,  /* MCA component major version */
-            OMPI_MINOR_VERSION,  /* MCA component minor version */
-            OMPI_RELEASE_VERSION,  /* MCA component release version */
-            component_open,
-            NULL,
-            NULL,
-            component_register
+            .mca_component_name = "portals4",
+            MCA_BASE_MAKE_VERSION(component, OMPI_MAJOR_VERSION, OMPI_MINOR_VERSION,
+                                  OMPI_RELEASE_VERSION),
+            .mca_open_component = component_open,
+            .mca_register_component_params = component_register,
         },
-        { /* mca_base_component_data */
+        .osc_data = {
             /* The component is not checkpoint ready */
             MCA_BASE_METADATA_PARAM_NONE
         },
-        component_init,
-        component_query,
-        component_select,
-        component_finalize
+        .osc_init = component_init,
+        .osc_query = component_query,
+        .osc_select = component_select,
+        .osc_finalize = component_finalize,
     }
 };
 
@@ -199,7 +198,7 @@ progress_callback(void)
                                 "%s:%d: PtlEQGet reported dropped event",
                                 __FILE__, __LINE__);
             goto process;
-        } else if (PTL_EQ_EMPTY) {
+        } else if (PTL_EQ_EMPTY == ret) {
             return 0;
         } else {
             opal_output_verbose(1, ompi_osc_base_framework.framework_output,
@@ -219,15 +218,18 @@ process:
         count++;
 
         if (NULL != ev.user_ptr) {
-            /* can't disable send events, but they don't count in ops */
-            if (ev.type == PTL_EVENT_SEND) continue;
+            /* be sure that we receive the PTL_EVENT_LINK */
+            if (ev.type == PTL_EVENT_LINK) {
+              *(int *)ev.user_ptr = *(int *)ev.user_ptr + 1;
+              opal_condition_broadcast(&mca_osc_portals4_component.cond);
+              continue;
+            }
+
             req = (ompi_osc_portals4_request_t*) ev.user_ptr;
             opal_atomic_add_size_t(&req->super.req_status._ucount, ev.mlength);
             ops = opal_atomic_add_32(&req->ops_committed, 1);
             if (ops == req->ops_expected) {
-                OPAL_THREAD_LOCK(&ompi_request_lock);
                 ompi_request_complete(&req->super, true);
-                OPAL_THREAD_UNLOCK(&ompi_request_lock);
             }
         }
     }
@@ -246,7 +248,7 @@ component_open(void)
 static int
 component_register(void)
 {
-    bool ompi_osc_portals4_no_locks = false;
+    mca_osc_portals4_component.no_locks = false;
     (void) mca_base_component_var_register(&mca_osc_portals4_component.super.osc_version,
                                            "no_locks",
                                            "Enable optimizations available only if MPI_LOCK is "
@@ -255,7 +257,19 @@ component_register(void)
                                            MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
                                            OPAL_INFO_LVL_9,
                                            MCA_BASE_VAR_SCOPE_READONLY,
-                                           &ompi_osc_portals4_no_locks);
+                                           &mca_osc_portals4_component.no_locks);
+
+    mca_osc_portals4_component.ptl_max_msg_size = PTL_SIZE_MAX;
+    (void) mca_base_component_var_register(&mca_osc_portals4_component.super.osc_version,
+                                           "max_msg_size",
+                                           "Max size supported by portals4 (above that, a message is cut into messages less than that size)",
+                                           MCA_BASE_VAR_TYPE_UNSIGNED_LONG,
+                                           NULL,
+                                           0,
+                                           0,
+                                           OPAL_INFO_LVL_9,
+                                           MCA_BASE_VAR_SCOPE_READONLY,
+                                           &mca_osc_portals4_component.ptl_max_msg_size);
 
     return OMPI_SUCCESS;
 }
@@ -290,9 +304,14 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
 
     /* BWB: FIX ME: Need to make sure our ID matches with the MTL... */
 
+    if (mca_osc_portals4_component.ptl_max_msg_size > actual.max_msg_size)
+        mca_osc_portals4_component.ptl_max_msg_size = actual.max_msg_size;
+    OPAL_OUTPUT_VERBOSE((10, ompi_osc_base_framework.framework_output,
+                         "max_size = %lu", mca_osc_portals4_component.ptl_max_msg_size));
+
     mca_osc_portals4_component.matching_atomic_max = actual.max_atomic_size;
     mca_osc_portals4_component.matching_fetch_atomic_max = actual.max_fetch_atomic_size;
-    mca_osc_portals4_component.matching_atomic_ordered_size = 
+    mca_osc_portals4_component.matching_atomic_ordered_size =
         MAX(actual.max_waw_ordered_size, actual.max_war_ordered_size);
 
     ret = PtlEQAlloc(mca_osc_portals4_component.matching_ni_h,
@@ -308,7 +327,7 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
     ret = PtlPTAlloc(mca_osc_portals4_component.matching_ni_h,
                      0,
                      mca_osc_portals4_component.matching_eq_h,
-                     4,
+                     REQ_OSC_TABLE_ID,
                      &mca_osc_portals4_component.matching_pt_idx);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
@@ -317,17 +336,22 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
         return ret;
     }
 
-    OBJ_CONSTRUCT(&mca_osc_portals4_component.requests, ompi_free_list_t);
-    ret = ompi_free_list_init(&mca_osc_portals4_component.requests,
-                              sizeof(ompi_osc_portals4_request_t),
-                              OBJ_CLASS(ompi_osc_portals4_request_t),
-                              8,
-                              0,
-                              8,
-                              NULL);
+    if (mca_osc_portals4_component.matching_pt_idx != REQ_OSC_TABLE_ID) {
+        opal_output_verbose(1, ompi_osc_base_framework.framework_output,
+                            "%s:%d: PtlPTAlloc did not allocate the requested PT: %d\n",
+                            __FILE__, __LINE__, mca_osc_portals4_component.matching_pt_idx);
+        return ret;
+    }
+
+    OBJ_CONSTRUCT(&mca_osc_portals4_component.requests, opal_free_list_t);
+    ret = opal_free_list_init (&mca_osc_portals4_component.requests,
+                               sizeof(ompi_osc_portals4_request_t),
+                               opal_cache_line_size,
+                               OBJ_CLASS(ompi_osc_portals4_request_t),
+                               0, 0, 8, 0, 8, NULL, 0, NULL, NULL, NULL);
     if (OMPI_SUCCESS != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                            "%s:%d: ompi_free_list_init failed: %d\n",
+                            "%s:%d: opal_free_list_init failed: %d\n",
                             __FILE__, __LINE__, ret);
         return ret;
     }
@@ -344,7 +368,7 @@ component_init(bool enable_progress_threads, bool enable_mpi_threads)
 }
 
 
-static int 
+static int
 component_finalize(void)
 {
     PtlNIFini(mca_osc_portals4_component.matching_ni_h);
@@ -358,8 +382,18 @@ component_query(struct ompi_win_t *win, void **base, size_t size, int disp_unit,
                 struct ompi_communicator_t *comm, struct ompi_info_t *info,
                 int flavor)
 {
+    int ret;
+
     if (MPI_WIN_FLAVOR_SHARED == flavor) return -1;
 
+    ret = PtlGetUid(mca_osc_portals4_component.matching_ni_h, &mca_osc_portals4_component.uid);
+    if (PTL_OK != ret) {
+      opal_output_verbose(1, ompi_osc_base_framework.framework_output,
+                          "%s:%d: PtlGetUid failed: %d\n",
+                          __FILE__, __LINE__, ret);
+      return OMPI_ERROR;
+    }
+
     return 20;
 }
 
@@ -384,7 +418,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     if (NULL == module) return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
 
     /* fill in the function pointer part */
-    memcpy(module, &ompi_osc_portals4_module_template, 
+    memcpy(module, &ompi_osc_portals4_module_template,
            sizeof(ompi_osc_base_module_t));
 
     /* fill in our part */
@@ -409,8 +443,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     /* share everyone's displacement units. Only do an allgather if
        strictly necessary, since it requires O(p) state. */
     tmp = disp_unit;
-    ret = module->comm->c_coll.coll_bcast(&tmp, 1, MPI_INT, 0, 
-                                          module->comm, 
+    ret = module->comm->c_coll.coll_bcast(&tmp, 1, MPI_INT, 0,
+                                          module->comm,
                                           module->comm->c_coll.coll_bcast_module);
     if (OMPI_SUCCESS != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
@@ -445,65 +479,13 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
                             __FILE__, __LINE__, ret);
         goto error;
     }
-    
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    {
-        int i;
-        int num_mds = ompi_mtl_portals4_get_num_mds();
-        ptl_size_t size = 1ULL << OMPI_PORTALS4_MAX_MD_SIZE;
-        ptl_size_t offset_unit = (1ULL << OMPI_PORTALS4_MAX_MD_SIZE) / 2;
 
-        module->md_h = malloc(sizeof(ptl_handle_md_t) * num_mds);
-        if (NULL == module->md_h) {
-            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-            goto error;
-        }
-        for (i = 0 ; i < num_mds ; ++i) {
-            module->md_h[i] = PTL_INVALID_HANDLE;
-        }
-
-        module->req_md_h = malloc(sizeof(ptl_handle_md_t) * num_mds);
-        if (NULL == module->req_md_h) {
-            ret = OMPI_ERR_TEMP_OUT_OF_RESOURCE;
-            goto error;
-        }
-        for (i = 0 ; i < num_mds ; ++i) {
-            module->req_md_h[i] = PTL_INVALID_HANDLE;
-        }
-
-        for (i = 0 ; i < num_mds ; ++i) {
-            md.start = (char*) (offset_unit * i);
-            md.length = (i - 1 == num_mds) ? size / 2 : size;
-
-            md.options = PTL_MD_EVENT_SUCCESS_DISABLE | PTL_MD_EVENT_CT_REPLY | PTL_MD_EVENT_CT_ACK;
-            md.eq_handle = mca_osc_portals4_component.matching_eq_h;
-            md.ct_handle = module->ct_h;
-            ret = PtlMDBind(module->ni_h, &md, &module->md_h);
-            if (PTL_OK != ret) {
-                opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                    "%s:%d: PtlMDBind failed: %d\n",
-                                    __FILE__, __LINE__, ret);
-                goto error;
-            }
-
-            md.options = PTL_MD_EVENT_CT_REPLY | PTL_MD_EVENT_CT_ACK;
-            md.eq_handle = mca_osc_portals4_component.matching_eq_h;
-            md.ct_handle = module->ct_h;
-            ret = PtlMDBind(module->ni_h, &md, &module->req_md_h);
-            if (PTL_OK != ret) {
-                opal_output_verbose(1, ompi_osc_base_framework.framework_output,
-                                    "%s:%d: PtlMDBind failed: %d\n",
-                                    __FILE__, __LINE__, ret);
-                goto error;
-            }
-        }
-#else
     md.start = 0;
     md.length = PTL_SIZE_MAX;
     md.options = PTL_MD_EVENT_SUCCESS_DISABLE | PTL_MD_EVENT_CT_REPLY | PTL_MD_EVENT_CT_ACK;
     md.eq_handle = mca_osc_portals4_component.matching_eq_h;
     md.ct_handle = module->ct_h;
-    ret = PtlMDBind(module->ni_h, &md, &module->md_h[0]);
+    ret = PtlMDBind(module->ni_h, &md, &module->md_h);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
                             "%s:%d: PtlMDBind failed: %d\n",
@@ -513,27 +495,26 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     md.start = 0;
     md.length = PTL_SIZE_MAX;
-    md.options = PTL_MD_EVENT_CT_REPLY | PTL_MD_EVENT_CT_ACK;
+    md.options = PTL_MD_EVENT_SEND_DISABLE | PTL_MD_EVENT_CT_REPLY | PTL_MD_EVENT_CT_ACK;
     md.eq_handle = mca_osc_portals4_component.matching_eq_h;
     md.ct_handle = module->ct_h;
-    ret = PtlMDBind(module->ni_h, &md, &module->req_md_h[0]);
+    ret = PtlMDBind(module->ni_h, &md, &module->req_md_h);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
                             "%s:%d: PtlMDBind failed: %d\n",
                             __FILE__, __LINE__, ret);
         goto error;
     }
-#endif
 
     if (MPI_WIN_FLAVOR_DYNAMIC == flavor) {
         me.start = 0;
-        me.length = SIZE_MAX;
+        me.length = PTL_SIZE_MAX;
     } else {
         me.start = *base;
         me.length = size;
     }
     me.ct_handle = PTL_CT_NONE;
-    me.uid = PTL_UID_ANY;
+    me.uid = mca_osc_portals4_component.uid;
     me.options = PTL_ME_OP_PUT | PTL_ME_OP_GET | PTL_ME_NO_TRUNCATE | PTL_ME_EVENT_SUCCESS_DISABLE;
     me.match_id.phys.nid = PTL_NID_ANY;
     me.match_id.phys.pid = PTL_PID_ANY;
@@ -544,7 +525,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
                       module->pt_idx,
                       &me,
                       PTL_PRIORITY_LIST,
-                      NULL,
+                      &module->ct_link,
                       &module->data_me_h);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
@@ -556,7 +537,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     me.start = &module->state;
     me.length = sizeof(module->state);
     me.ct_handle = PTL_CT_NONE;
-    me.uid = PTL_UID_ANY;
+    me.uid = mca_osc_portals4_component.uid;
     me.options = PTL_ME_OP_PUT | PTL_ME_OP_GET | PTL_ME_NO_TRUNCATE | PTL_ME_EVENT_SUCCESS_DISABLE;
     me.match_id.phys.nid = PTL_NID_ANY;
     me.match_id.phys.pid = PTL_PID_ANY;
@@ -567,7 +548,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
                       module->pt_idx,
                       &me,
                       PTL_PRIORITY_LIST,
-                      NULL,
+                      &module->ct_link,
                       &module->control_me_h);
     if (PTL_OK != ret) {
         opal_output_verbose(1, ompi_osc_base_framework.framework_output,
@@ -604,7 +585,7 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
 
     module->passive_target_access_epoch = false;
 
-#if OPAL_ASSEMBLY_ARCH == OMPI_AMD64 || OPAL_ASSEMBLY_ARCH == IA32
+#if OPAL_ASSEMBLY_ARCH == OPAL_X86_64 || OPAL_ASSEMBLY_ARCH == OPAL_IA32
     *model = MPI_WIN_UNIFIED;
 #else
     *model = MPI_WIN_SEPARATE;
@@ -615,6 +596,13 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     PtlAtomicSync();
 
     /* Make sure that everyone's ready to receive. */
+    OPAL_THREAD_LOCK(&mca_osc_portals4_component.lock);
+    while (module->ct_link != 2) {
+        opal_condition_wait(&mca_osc_portals4_component.cond,
+                            &mca_osc_portals4_component.lock);
+    }
+    OPAL_THREAD_UNLOCK(&mca_osc_portals4_component.lock);
+
     module->comm->c_coll.coll_barrier(module->comm,
                                       module->comm->c_coll.coll_barrier_module);
 
@@ -624,12 +612,8 @@ component_select(struct ompi_win_t *win, void **base, size_t size, int disp_unit
     /* BWB: FIX ME: This is all wrong... */
     if (0 != module->ct_h) PtlCTFree(module->ct_h);
     if (0 != module->data_me_h) PtlMEUnlink(module->data_me_h);
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    /* BWB: FIX ME */
-#else 
-    if (0 != module->req_md_h) PtlMDRelease(module->req_md_h[0]);
-    if (0 != module->md_h) PtlMDRelease(module->md_h[0]);
-#endif
+    if (0 != module->req_md_h) PtlMDRelease(module->req_md_h);
+    if (0 != module->md_h) PtlMDRelease(module->md_h);
     if (NULL != module->comm) ompi_comm_free(&module->comm);
     if (NULL != module) free(module);
 
@@ -663,13 +647,10 @@ ompi_osc_portals4_free(struct ompi_win_t *win)
                                       module->comm->c_coll.coll_barrier_module);
 
     /* cleanup */
+    PtlMEUnlink(module->control_me_h);
     PtlMEUnlink(module->data_me_h);
-#if OMPI_PORTALS4_MAX_MD_SIZE < OMPI_PORTALS4_MAX_VA_SIZE
-    /* BWB: FIX ME */
-#else 
-    PtlMDRelease(module->md_h[0]);
-    PtlMDRelease(module->req_md_h[0]);
-#endif
+    PtlMDRelease(module->md_h);
+    PtlMDRelease(module->req_md_h);
     PtlCTFree(module->ct_h);
     if (NULL != module->disp_units) free(module->disp_units);
     ompi_comm_free(&module->comm);
