@@ -14,6 +14,8 @@
  * Copyright (c) 2012      Oak Ridge National Laboratory. All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2014-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,12 +33,11 @@
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/memchecker.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_Ialltoall = PMPI_Ialltoall
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/c/profile/defines.h"
+#define MPI_Ialltoall PMPI_Ialltoall
 #endif
 
 static const char FUNC_NAME[] = "MPI_Ialltoall";
@@ -46,43 +47,53 @@ int MPI_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype,
                   void *recvbuf, int recvcount, MPI_Datatype recvtype,
                   MPI_Comm comm,  MPI_Request *request)
 {
+    size_t sendtype_size, recvtype_size;
     int err;
 
     MEMCHECKER(
-        memchecker_datatype(sendtype);
-        memchecker_datatype(recvtype);
-        memchecker_call(&opal_memchecker_base_isdefined, sendbuf, sendcount, sendtype);
-        memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
         memchecker_comm(comm);
+        if (MPI_IN_PLACE != sendbuf) {
+            memchecker_datatype(sendtype);
+            memchecker_call(&opal_memchecker_base_isdefined, (void *)sendbuf, sendcount, sendtype);
+        }
+        memchecker_datatype(recvtype);
+        memchecker_call(&opal_memchecker_base_isaddressable, recvbuf, recvcount, recvtype);
     );
 
     if (MPI_PARAM_CHECK) {
 
-      /* Unrooted operation -- same checks for all ranks on both
-         intracommunicators and intercommunicators */
+        /* Unrooted operation -- same checks for all ranks on both
+           intracommunicators and intercommunicators */
 
-      err = MPI_SUCCESS;
-      OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
-      if (ompi_comm_invalid(comm)) {
-        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
-                                     FUNC_NAME);
-      } else if (MPI_DATATYPE_NULL == recvtype || NULL == recvtype) {
-        err = MPI_ERR_TYPE;
-      } else if (recvcount < 0) {
-        err = MPI_ERR_COUNT;
-      } else if (MPI_IN_PLACE == sendbuf || MPI_IN_PLACE == recvbuf) {
-        err = MPI_ERR_ARG;
-      } else {
-        OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcount);
-      }
-      OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+        err = MPI_SUCCESS;
+        OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
+        if (ompi_comm_invalid(comm)) {
+            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+                                          FUNC_NAME);
+        } else if ((MPI_IN_PLACE == sendbuf && OMPI_COMM_IS_INTER(comm)) ||
+                   MPI_IN_PLACE == recvbuf) {
+            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_ARG,
+                                          FUNC_NAME);
+        } else {
+            if (MPI_IN_PLACE != sendbuf) {
+                OMPI_CHECK_DATATYPE_FOR_SEND(err, sendtype, sendcount);
+                OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+            }
+            OMPI_CHECK_DATATYPE_FOR_RECV(err, recvtype, recvcount);
+            OMPI_ERRHANDLER_CHECK(err, comm, err, FUNC_NAME);
+        }
+
+        if (MPI_IN_PLACE != sendbuf && !OMPI_COMM_IS_INTER(comm)) {
+            ompi_datatype_type_size(sendtype, &sendtype_size);
+            ompi_datatype_type_size(recvtype, &recvtype_size);
+            if ((sendtype_size*sendcount) != (recvtype_size*recvcount)) {
+                return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_TRUNCATE, FUNC_NAME);
+            }
+        }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Invoke the coll component to perform the back-end operation */
-    /* XXX -- CONST -- do not cast away const -- update mca/coll */
-    err = comm->c_coll.coll_ialltoall((void *) sendbuf, sendcount, sendtype,
+    err = comm->c_coll.coll_ialltoall(sendbuf, sendcount, sendtype,
                                       recvbuf, recvcount, recvtype, comm,
                                       request, comm->c_coll.coll_ialltoall_module);
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
