@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,20 +6,20 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2011-2012 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
+ * Copyright (c) 2011-2017 Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2012-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
  * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
- * Copyright (c) 2015-2016 Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -40,7 +41,7 @@
 #include "opal/util/show_help.h"
 #include "opal/threads/tsd.h"
 
-#include "opal/mca/hwloc/hwloc.h"
+#include "opal/mca/hwloc/hwloc-internal.h"
 #include "opal/mca/hwloc/base/base.h"
 
 /*
@@ -56,10 +57,10 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
 {
     hwloc_obj_type_t obj_type = HWLOC_OBJ_CORE;
     hwloc_obj_t obj;
-    
+
     /* hwloc isn't able to find cores on all platforms.  Example:
        PPC64 running RHEL 5.4 (linux kernel 2.6.18) only reports NUMA
-       nodes and PU's.  Fine.  
+       nodes and PU's.  Fine.
 
        However, note that hwloc_get_obj_by_type() will return NULL in
        2 (effectively) different cases:
@@ -90,10 +91,10 @@ hwloc_obj_t opal_hwloc_base_get_pu(hwloc_topology_t topo,
         }
         return obj;
     }
-            
+
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "Searching for %d LOGICAL PU", lid);
-    
+
     /* Now do the actual lookup. */
     obj = hwloc_get_obj_by_type(topo, obj_type, lid);
     OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
@@ -246,7 +247,7 @@ int opal_hwloc_base_get_topology(void)
 
     if (NULL == opal_hwloc_base_topo_file) {
         if (0 != hwloc_topology_init(&opal_hwloc_topology) ||
-            0 != hwloc_topology_set_flags(opal_hwloc_topology, 
+            0 != hwloc_topology_set_flags(opal_hwloc_topology,
                                           (HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM |
                                            HWLOC_TOPOLOGY_FLAG_IO_DEVICES)) ||
             0 != hwloc_topology_load(opal_hwloc_topology)) {
@@ -264,13 +265,12 @@ int opal_hwloc_base_get_topology(void)
     /* fill opal_cache_line_size global with the smallest L1 cache
        line size */
     fill_cache_line_size();
+
     return rc;
 }
 
 int opal_hwloc_base_set_topology(char *topofile)
 {
-    hwloc_obj_t obj;
-    unsigned j, k;
     struct hwloc_topology_support *support;
     int rc;
 
@@ -305,30 +305,7 @@ int opal_hwloc_base_set_topology(char *topofile)
                              "hwloc:base:set_topology failed to load"));
         return OPAL_ERR_NOT_SUPPORTED;
     }
-    /* remove the hostname from the topology. Unfortunately, hwloc
-     * decided to add the source hostname to the "topology", thus
-     * rendering it unusable as a pure topological description. So
-     * we remove that information here.
-     */
-    obj = hwloc_get_root_obj(opal_hwloc_topology);
-    for (k=0; k < obj->infos_count; k++) {
-        if (NULL == obj->infos[k].name ||
-            NULL == obj->infos[k].value) {
-            continue;
-        }
-        if (0 == strncmp(obj->infos[k].name, "HostName", strlen("HostName"))) {
-            free(obj->infos[k].name);
-            free(obj->infos[k].value);
-            /* left justify the array */
-            for (j=k; j < obj->infos_count-1; j++) {
-                obj->infos[j] = obj->infos[j+1];
-            }
-            obj->infos[obj->infos_count-1].name = NULL;
-            obj->infos[obj->infos_count-1].value = NULL;
-            obj->infos_count--;
-            break;
-        }
-    }
+
     /* unfortunately, hwloc does not include support info in its
      * xml output :-(( We default to assuming it is present as
      * systems that use this option are likely to provide
@@ -403,8 +380,8 @@ void opal_hwloc_base_get_local_cpuset(void)
         }
 
         /* get the cpus we are bound to */
-        if (hwloc_get_cpubind(opal_hwloc_topology, 
-                              opal_hwloc_my_cpuset, 
+        if (hwloc_get_cpubind(opal_hwloc_topology,
+                              opal_hwloc_my_cpuset,
                               HWLOC_CPUBIND_PROCESS) < 0) {
             /* we are not bound - use the root's available cpuset */
             root = hwloc_get_root_obj(opal_hwloc_topology);
@@ -422,7 +399,7 @@ int opal_hwloc_base_report_bind_failure(const char *file,
 
     if (!already_reported &&
         OPAL_HWLOC_BASE_MBFA_SILENT != opal_hwloc_base_mbfa) {
-        char hostname[64];
+        char hostname[OPAL_MAXHOSTNAMELEN];
         gethostname(hostname, sizeof(hostname));
 
         opal_show_help("help-opal-hwloc-base.txt", "mbind failure", true,
@@ -790,7 +767,7 @@ static hwloc_obj_t df_search(hwloc_topology_t topo,
             return obj;
         }
     }
-    
+
     return NULL;
 }
 
@@ -895,7 +872,7 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
             data = OBJ_NEW(opal_hwloc_obj_data_t);
             start->userdata = data;
         }
-            
+
         OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                              "hwloc:base:min_bound_under_obj object %s:%u nbound %u min %u",
                              hwloc_obj_type_string(target), start->logical_index,
@@ -923,7 +900,7 @@ static hwloc_obj_t df_search_min_bound(hwloc_topology_t topo,
             break;
         }
     }
-    
+
     return save;
 }
 
@@ -1089,7 +1066,7 @@ static int socket_to_cpu_set(char *cpus,
         res = opal_hwloc_base_get_available_cpus(topo, obj);
         hwloc_bitmap_or(cpumask, cpumask, res);
         break;
-                
+
     case 2:  /* range of sockets was given */
         lower_range = atoi(range[0]);
         upper_range = atoi(range[1]);
@@ -1129,7 +1106,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
 
     socket_core = opal_argv_split(socket_core_list, ':');
     socket_id = atoi(socket_core[0]);
-    
+
     /* get the object for this socket id */
     if (NULL == (socket = opal_hwloc_base_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0,
                                                           socket_id, rtype))) {
@@ -1173,6 +1150,9 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     if (NULL == (core = df_search(topo, socket, obj_type, 0,
                                                   core_id, OPAL_HWLOC_AVAILABLE,
                                                   &idx, NULL))) {
+                        opal_argv_free(list);
+                        opal_argv_free(range);
+                        opal_argv_free(socket_core);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     /* get the cpus */
@@ -1181,7 +1161,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                 }
                 opal_argv_free(list);
                 break;
-                
+
             case 2:  /* range of core id's was given */
                 opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                                     "range of cores given: start %s stop %s",
@@ -1194,6 +1174,8 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     if (NULL == (core = df_search(topo, socket, obj_type, 0,
                                                   core_id, OPAL_HWLOC_AVAILABLE,
                                                   &idx, NULL))) {
+                        opal_argv_free(range);
+                        opal_argv_free(socket_core);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     /* get the cpus */
@@ -1202,7 +1184,7 @@ static int socket_core_to_cpu_set(char *socket_core_list,
                     hwloc_bitmap_or(cpumask, cpumask, res);
                 }
                 break;
-                
+
             default:
                 opal_argv_free(range);
                 opal_argv_free(socket_core);
@@ -1236,14 +1218,14 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
     if (NULL == slot_str || 0 == strlen(slot_str)) {
         return OPAL_ERR_BAD_PARAM;
     }
-    
+
     opal_output_verbose(5, opal_hwloc_base_framework.framework_output,
                         "slot assignment: slot_list == %s",
                         slot_str);
 
     /* split at ';' */
     item = opal_argv_split(slot_str, ';');
-    
+
     /* start with a clean mask */
     hwloc_bitmap_zero(cpumask);
     /* loop across the items and accumulate the mask */
@@ -1315,6 +1297,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                             opal_argv_free(range);
                             opal_argv_free(item);
                             opal_argv_free(rngs);
+                            opal_argv_free(list);
                             return OPAL_ERR_SILENT;
                         }
                         /* get the available cpus for that object */
@@ -1324,7 +1307,7 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                     }
                     opal_argv_free(list);
                     break;
-                    
+
                 case 2:  /* range of core id's was given */
                     lower_range = atoi(range[0]);
                     upper_range = atoi(range[1]);
@@ -1342,13 +1325,14 @@ int opal_hwloc_base_slot_list_parse(const char *slot_str,
                         hwloc_bitmap_or(cpumask, cpumask, pucpus);
                     }
                     break;
-                
+
                 default:
                     opal_argv_free(range);
                     opal_argv_free(item);
                     opal_argv_free(rngs);
                     return OPAL_ERROR;
                 }
+                opal_argv_free(range);
             }
             opal_argv_free(rngs);
         }
@@ -1373,7 +1357,7 @@ opal_hwloc_locality_t opal_hwloc_base_get_relative_locality(hwloc_topology_t top
      * NOTE: we may alter that latter part as hwloc's ability to
      * sense multi-cu, multi-cluster systems grows
      */
-    locality = OPAL_PROC_ON_NODE;
+    locality = OPAL_PROC_ON_NODE | OPAL_PROC_ON_HOST | OPAL_PROC_ON_CU | OPAL_PROC_ON_CLUSTER;
 
     /* if either cpuset is NULL, then that isn't bound */
     if (NULL == cpuset1 || NULL == cpuset2) {
@@ -1421,25 +1405,25 @@ opal_hwloc_locality_t opal_hwloc_base_get_relative_locality(hwloc_topology_t top
                 shared = true;
                 switch(obj->type) {
                 case HWLOC_OBJ_NODE:
-                    locality = OPAL_PROC_ON_NUMA;
+                    locality |= OPAL_PROC_ON_NUMA;
                     break;
                 case HWLOC_OBJ_SOCKET:
-                    locality = OPAL_PROC_ON_SOCKET;
+                    locality |= OPAL_PROC_ON_SOCKET;
                     break;
                 case HWLOC_OBJ_CACHE:
                     if (3 == obj->attr->cache.depth) {
-                        locality = OPAL_PROC_ON_L3CACHE;
+                        locality |= OPAL_PROC_ON_L3CACHE;
                     } else if (2 == obj->attr->cache.depth) {
-                        locality = OPAL_PROC_ON_L2CACHE;
+                        locality |= OPAL_PROC_ON_L2CACHE;
                     } else {
-                        locality = OPAL_PROC_ON_L1CACHE;
+                        locality |= OPAL_PROC_ON_L1CACHE;
                     }
                     break;
                 case HWLOC_OBJ_CORE:
-                    locality = OPAL_PROC_ON_CORE;
+                    locality |= OPAL_PROC_ON_CORE;
                     break;
                 case HWLOC_OBJ_PU:
-                    locality = OPAL_PROC_ON_HWTHREAD;
+                    locality |= OPAL_PROC_ON_HWTHREAD;
                     break;
                 default:
                     /* just ignore it */
@@ -1490,6 +1474,7 @@ char* opal_hwloc_base_find_coprocessors(hwloc_topology_t topo)
                              "hwloc:base:find_coprocessors: NONE FOUND IN TOPO"));
         return NULL;
     }
+#if HAVE_DECL_HWLOC_OBJ_OSDEV_COPROC
     /* check the device objects for coprocessors */
     osdev = hwloc_get_obj_by_depth(topo, depth, 0);
     while (NULL != osdev) {
@@ -1513,6 +1498,11 @@ char* opal_hwloc_base_find_coprocessors(hwloc_topology_t topo)
     OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
                          "hwloc:base:find_coprocessors: hosting coprocessors %s",
                          (NULL == cpstring) ? "NONE" : cpstring));
+#else
+    OPAL_OUTPUT_VERBOSE((5, opal_hwloc_base_framework.framework_output,
+                         "hwloc:base:find_coprocessors: the version of hwloc that Open MPI was built against (v%d.%d.%d) does not support detecting coprocessors",
+                         (HWLOC_API_VERSION>>16)&&0xFF, (HWLOC_API_VERSION>>8)&0xFF, HWLOC_API_VERSION&&0xFF));
+#endif
     return cpstring;
 }
 
@@ -1529,7 +1519,7 @@ static char *hwloc_getline(FILE *fp)
 	   buff = strdup(input);
 	   return buff;
     }
-    
+
     return NULL;
 }
 
@@ -1678,6 +1668,7 @@ static char *bitmap2rangestr(int bitmap)
                 /* A range just ended; output it */
                 if (!first) {
                     strncat(ret, ",", sizeof(ret) - strlen(ret) - 1);
+                } else {
                     first = false;
                 }
 
@@ -1723,7 +1714,7 @@ static char *bitmap2rangestr(int bitmap)
 /*
  * Make a map of socket/core/hwthread tuples
  */
-static int build_map(int *num_sockets_arg, int *num_cores_arg, 
+static int build_map(int *num_sockets_arg, int *num_cores_arg,
                      hwloc_cpuset_t cpuset, int ***map, hwloc_topology_t topo)
 {
     int num_sockets, num_cores;
@@ -1764,11 +1755,11 @@ static int build_map(int *num_sockets_arg, int *num_cores_arg,
        the socket/core/pu triples */
     for (pu_index = 0,
              pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                      cpuset, HWLOC_OBJ_PU, 
+                                                      cpuset, HWLOC_OBJ_PU,
                                                       pu_index);
          NULL != pu;
          pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                  cpuset, HWLOC_OBJ_PU, 
+                                                  cpuset, HWLOC_OBJ_PU,
                                                   ++pu_index)) {
         /* Go upward and find the core this PU belongs to */
         core = pu;
@@ -1826,13 +1817,14 @@ int opal_hwloc_base_cset2str(char *str, int len,
     root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
         opal_hwloc_base_filter_cpus(topo);
-    }
-    sum = (opal_hwloc_topo_data_t*)root->userdata;
-    if (NULL == sum->available) {
-       return OPAL_ERROR;
-    }
-    if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
-        return OPAL_ERR_NOT_BOUND;
+    } else {
+        sum = (opal_hwloc_topo_data_t*)root->userdata;
+        if (NULL == sum->available) {
+           return OPAL_ERROR;
+        }
+        if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
+            return OPAL_ERR_NOT_BOUND;
+        }
     }
 
     if (OPAL_SUCCESS != (ret = build_map(&num_sockets, &num_cores, cpuset, &map, topo))) {
@@ -1848,7 +1840,7 @@ int opal_hwloc_base_cset2str(char *str, int len,
                 }
                 first = false;
 
-                snprintf(tmp, stmp, "socket %d[core %d[hwt %s]]", 
+                snprintf(tmp, stmp, "socket %d[core %d[hwt %s]]",
                          socket_index, core_index,
                          bitmap2rangestr(map[socket_index][core_index]));
                 strncat(str, tmp, len - strlen(str));
@@ -1866,7 +1858,7 @@ int opal_hwloc_base_cset2str(char *str, int len,
 }
 
 /*
- * Make a prettyprint string for a cset in a map format.  
+ * Make a prettyprint string for a cset in a map format.
  * Example: [B./..]
  * Key:  [] - signifies socket
  *        / - divider between cores
@@ -1895,29 +1887,30 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
     root = hwloc_get_root_obj(topo);
     if (NULL == root->userdata) {
         opal_hwloc_base_filter_cpus(topo);
-    }
-    sum = (opal_hwloc_topo_data_t*)root->userdata;
-    if (NULL == sum->available) {
-       return OPAL_ERROR;
-    }
-    if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
-        return OPAL_ERR_NOT_BOUND;
+    } else {
+        sum = (opal_hwloc_topo_data_t*)root->userdata;
+        if (NULL == sum->available) {
+           return OPAL_ERROR;
+        }
+        if (0 != hwloc_bitmap_isincluded(sum->available, cpuset)) {
+            return OPAL_ERR_NOT_BOUND;
+        }
     }
 
     /* Iterate over all existing sockets */
     for (socket = hwloc_get_obj_by_type(topo, HWLOC_OBJ_SOCKET, 0);
-         NULL != socket; 
+         NULL != socket;
          socket = socket->next_cousin) {
         strncat(str, "[", len - strlen(str));
 
         /* Iterate over all existing cores in this socket */
         core_index = 0;
         for (core = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                        socket->cpuset, 
+                                                        socket->cpuset,
                                                         HWLOC_OBJ_CORE, core_index);
-             NULL != core; 
+             NULL != core;
              core = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                        socket->cpuset, 
+                                                        socket->cpuset,
                                                         HWLOC_OBJ_CORE, ++core_index)) {
             if (core_index > 0) {
                 strncat(str, "/", len - strlen(str));
@@ -1926,11 +1919,11 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
             /* Iterate over all existing PUs in this core */
             pu_index = 0;
             for (pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                          core->cpuset, 
+                                                          core->cpuset,
                                                           HWLOC_OBJ_PU, pu_index);
-                 NULL != pu; 
+                 NULL != pu;
                  pu = hwloc_get_obj_inside_cpuset_by_type(topo,
-                                                          core->cpuset, 
+                                                          core->cpuset,
                                                           HWLOC_OBJ_PU, ++pu_index)) {
 
                 /* Is this PU in the cpuset? */
@@ -1949,8 +1942,8 @@ int opal_hwloc_base_cset2mapstr(char *str, int len,
 
 static int dist_cmp_fn (opal_list_item_t **a, opal_list_item_t **b)
 {
-    orte_rmaps_numa_node_t *aitem = *((orte_rmaps_numa_node_t **) a);
-    orte_rmaps_numa_node_t *bitem = *((orte_rmaps_numa_node_t **) b);
+    opal_rmaps_numa_node_t *aitem = *((opal_rmaps_numa_node_t **) a);
+    opal_rmaps_numa_node_t *bitem = *((opal_rmaps_numa_node_t **) b);
 
     if (aitem->dist_from_closed > bitem->dist_from_closed) {
         return 1;
@@ -1966,7 +1959,7 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
     hwloc_obj_t device_obj = NULL;
     hwloc_obj_t obj = NULL, root = NULL;
     const struct hwloc_distances_s* distances;
-    orte_rmaps_numa_node_t *numa_node;
+    opal_rmaps_numa_node_t *numa_node;
     int close_node_index;
     float latency;
     unsigned int j;
@@ -2023,7 +2016,7 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
                 /* fill list of numa nodes */
                 for (j = 0; j < distances->nbobjs; j++) {
                     latency = distances->latency[close_node_index + distances->nbobjs * j];
-                    numa_node = OBJ_NEW(orte_rmaps_numa_node_t);
+                    numa_node = OBJ_NEW(opal_rmaps_numa_node_t);
                     numa_node->index = j;
                     numa_node->dist_from_closed = latency;
                     opal_list_append(sorted_list, &numa_node->super);
@@ -2036,15 +2029,15 @@ static void sort_by_dist(hwloc_topology_t topo, char* device_name, opal_list_t *
     }
 }
 
-static int find_devices(hwloc_topology_t topo, char* device_name) 
+static int find_devices(hwloc_topology_t topo, char** device_name)
 {
     hwloc_obj_t device_obj = NULL;
     int count = 0;
     for (device_obj = hwloc_get_obj_by_type(topo, HWLOC_OBJ_OS_DEVICE, 0); device_obj; device_obj = hwloc_get_next_osdev(topo, device_obj)) {
         if (device_obj->attr->osdev.type == HWLOC_OBJ_OSDEV_OPENFABRICS) {
             count++;
-            free(device_name);
-            device_name = strdup(device_obj->name);
+            free(*device_name);
+            *device_name = strdup(device_obj->name);
         }
     }
     return count;
@@ -2056,7 +2049,7 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
     opal_list_item_t *item;
     opal_hwloc_summary_t *sum;
     opal_hwloc_topo_data_t *data;
-    orte_rmaps_numa_node_t *numa, *copy_numa;
+    opal_rmaps_numa_node_t *numa, *copy_numa;
     int count;
 
     obj = hwloc_get_root_obj(topo);
@@ -2070,9 +2063,9 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
                 item = opal_list_get_next(item)) {
             sum = (opal_hwloc_summary_t*)item;
             if (HWLOC_OBJ_NODE == sum->type) {
-                if (opal_list_get_size(&sum->sorted_by_dist_list) > 0) { 
-                    OPAL_LIST_FOREACH(numa, &(sum->sorted_by_dist_list), orte_rmaps_numa_node_t) {
-                        copy_numa = OBJ_NEW(orte_rmaps_numa_node_t);
+                if (opal_list_get_size(&sum->sorted_by_dist_list) > 0) {
+                    OPAL_LIST_FOREACH(numa, &(sum->sorted_by_dist_list), opal_rmaps_numa_node_t) {
+                        copy_numa = OBJ_NEW(opal_rmaps_numa_node_t);
                         copy_numa->index = numa->index;
                         copy_numa->dist_from_closed = numa->dist_from_closed;
                         opal_list_append(sorted_list, &copy_numa->super);
@@ -2082,18 +2075,22 @@ int opal_hwloc_get_sorted_numa_list(hwloc_topology_t topo, char* device_name, op
                     /* don't already know it - go get it */
                     /* firstly we check if we need to autodetect OpenFabrics  devices or we have the specified one */
                     if (!strcmp(device_name, "auto")) {
-                        count = find_devices(topo, device_name);
-                       if (count > 1) {
-                           return count;
-                       }
+                        count = find_devices(topo, &device_name);
+                        if (count > 1) {
+                            free(device_name);
+                            return count;
+                        }
                     }
-                    if (!device_name || (strlen(device_name) == 0)) {
+                    if (!device_name) {
+                        return OPAL_ERR_NOT_FOUND;
+                    } else if (strlen(device_name) == 0) {
+                        free(device_name);
                         return OPAL_ERR_NOT_FOUND;
                     }
                     sort_by_dist(topo, device_name, sorted_list);
                     /* store this info in summary object for later usage */
-                    OPAL_LIST_FOREACH(numa, sorted_list, orte_rmaps_numa_node_t) {
-                        copy_numa = OBJ_NEW(orte_rmaps_numa_node_t);
+                    OPAL_LIST_FOREACH(numa, sorted_list, opal_rmaps_numa_node_t) {
+                        copy_numa = OBJ_NEW(opal_rmaps_numa_node_t);
                         copy_numa->index = numa->index;
                         copy_numa->dist_from_closed = numa->dist_from_closed;
                         opal_list_append(&(sum->sorted_by_dist_list), &copy_numa->super);
@@ -2112,7 +2109,7 @@ char* opal_hwloc_base_get_topo_signature(hwloc_topology_t topo)
     char *sig=NULL, *arch=NULL;
     hwloc_obj_t obj;
     unsigned i;
-    
+
     nnuma = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_NODE, 0, OPAL_HWLOC_AVAILABLE);
     nsocket = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_SOCKET, 0, OPAL_HWLOC_AVAILABLE);
     nl3 = opal_hwloc_base_get_nbobjs_by_type(topo, HWLOC_OBJ_CACHE, 3, OPAL_HWLOC_AVAILABLE);
