@@ -10,9 +10,12 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ *                         reserved.
  * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -35,6 +38,7 @@
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/util/if.h"
+#include "opal/util/net.h"
 #include "opal/mca/installdirs/installdirs.h"
 
 #include "orte/util/show_help.h"
@@ -56,9 +60,9 @@ static void hostfile_parse_error(int token)
     case ORTE_HOSTFILE_STRING:
         orte_show_help("help-hostfile.txt", "parse_error_string",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
-                       token, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
+                       token,
                        orte_util_hostfile_value.sval);
         break;
     case ORTE_HOSTFILE_IPV4:
@@ -66,16 +70,16 @@ static void hostfile_parse_error(int token)
     case ORTE_HOSTFILE_INT:
         orte_show_help("help-hostfile.txt", "parse_error_int",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
-                       token, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
+                       token,
                        orte_util_hostfile_value.ival);
         break;
      default:
         orte_show_help("help-hostfile.txt", "parse_error",
                        true,
-                       cur_hostfile_name, 
-                       orte_util_hostfile_line, 
+                       cur_hostfile_name,
+                       orte_util_hostfile_line,
                        token );
         break;
     }
@@ -109,17 +113,14 @@ static char *hostfile_parse_string(void)
     return strdup(orte_util_hostfile_value.sval);
 }
 
-static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name, bool keep)
+static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name)
 {
     opal_list_item_t* item;
     for(item =  opal_list_get_first(nodes);
         item != opal_list_get_end(nodes);
         item =  opal_list_get_next(item)) {
         orte_node_t* node = (orte_node_t*)item;
-        if(strcmp(node->name, name) == 0) {
-            if (!keep) {
-                opal_list_remove_item(nodes, item);
-            }
+        if (strcmp(node->name, name) == 0) {
             return node;
         }
     }
@@ -133,9 +134,8 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
     orte_node_t* node;
     bool got_max = false;
     char* value;
-    char** argv;
+    char **argv;
     char* node_name = NULL;
-    char* node_alias = NULL;
     char* username = NULL;
     int cnt;
     int number_of_slots = 0;
@@ -154,18 +154,26 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
             value = orte_util_hostfile_value.sval;
         }
         argv = opal_argv_split (value, '@');
-        
+
         cnt = opal_argv_count (argv);
         if (1 == cnt) {
             node_name = strdup(argv[0]);
         } else if (2 == cnt) {
-            username = strdup(argv[0]);
+            username = argv[0];
             node_name = strdup(argv[1]);
         } else {
             opal_output(0, "WARNING: Unhandled user@host-combination\n"); /* XXX */
         }
         opal_argv_free (argv);
 
+        // Strip off the FQDN if present, ignore IP addresses
+        if( !orte_keep_fqdn_hostnames && !opal_net_isaddr(node_name) ) {
+            char *ptr;
+            if (NULL != (ptr = strchr(node_name, '.'))) {
+                *ptr = '\0';
+            }
+        }
+
         /* if the first letter of the name is '^', then this is a node
          * to be excluded. Remove the ^ character so the nodename is
          * usable, and put it on the exclude list
@@ -177,48 +185,38 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
                 node_name[i-1] = node_name[i];
             }
             node_name[len-1] = '\0';  /* truncate */
-            
+
             OPAL_OUTPUT_VERBOSE((3, orte_ras_base_framework.framework_output,
                                  "%s hostfile: node %s is being excluded",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node_name));
-            
-            /* convert this into something globally unique */
-            if (strcmp(node_name, "localhost") == 0 || opal_ifislocal(node_name)) {
+
+            /* see if this is another name for us */
+            if (orte_ifislocal(node_name)) {
                 /* Nodename has been allocated, that is for sure */
-                if (orte_show_resolved_nodenames &&
-                    0 != strcmp(node_name, orte_process_info.nodename)) {
-                    node_alias = strdup(node_name);
-                }
                 free (node_name);
                 node_name = strdup(orte_process_info.nodename);
             }
-            
+
             /* Do we need to make a new node object?  First check to see
                if it's already in the exclude list */
-            if (NULL == (node = hostfile_lookup(exclude, node_name, keep_all))) {
+            if (NULL == (node = hostfile_lookup(exclude, node_name))) {
                 node = OBJ_NEW(orte_node_t);
                 node->name = node_name;
                 if (NULL != username) {
-                    node->username = strdup(username);
+                    orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
                 }
+                opal_list_append(exclude, &node->super);
+            } else {
+                free(node_name);
             }
-            /* Note that we need to add this back to the exclude list.
-               If it was found, we just removed it (in hostfile_lookup()),
-               so this puts it back. If it was not found, then we have to
-               add it to the exclude list anyway. */
-            opal_list_append(exclude, &node->super);
             return ORTE_SUCCESS;
         }
-        
+
         /* this is not a node to be excluded, so we need to process it and
          * add it to the "include" list. See if this host is actually us.
          */
-        if (strcmp(node_name, "localhost") == 0 || opal_ifislocal(node_name)) {
+        if (orte_ifislocal(node_name)) {
             /* Nodename has been allocated, that is for sure */
-            if (orte_show_resolved_nodenames &&
-                0 != strcmp(node_name, orte_process_info.nodename)) {
-                node_alias = strdup(node_name);
-            }
             free (node_name);
             node_name = strdup(orte_process_info.nodename);
         }
@@ -229,31 +227,25 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
                              keep_all ? "TRUE" : "FALSE"));
 
         /* Do we need to make a new node object? */
-        if (NULL == (node = hostfile_lookup(updates, node_name, keep_all))) {
+        if (keep_all || NULL == (node = hostfile_lookup(updates, node_name))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
             node->slots = 1;
             if (NULL != username) {
-                node->username = strdup(username);
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
             }
+            opal_list_append(updates, &node->super);
         } else {
             /* this node was already found once - add a slot and mark slots as "given" */
             node->slots++;
-            node->slots_given = true;
-        }
-        /* do we need to record an alias for this node? */
-        if (NULL != node_alias) {
-            /* add to list of aliases for this node - only add if unique */
-            opal_argv_append_unique_nosize(&node->alias, node_alias, false);
-            free(node_alias);
+            ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
+            free(node_name);
         }
     } else if (ORTE_HOSTFILE_RELATIVE == token) {
         /* store this for later processing */
         node = OBJ_NEW(orte_node_t);
         node->name = strdup(orte_util_hostfile_value.sval);
-        if (NULL != username) {
-            node->username = strdup(username);
-        }
+        opal_list_append(updates, &node->super);
     } else if (ORTE_HOSTFILE_RANK == token) {
         /* we can ignore the rank, but we need to extract the node name. we
          * first need to shift over to the other side of the equal sign as
@@ -275,30 +267,41 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
         } else {
             value = orte_util_hostfile_value.sval;
         }
-        
+
         argv = opal_argv_split (value, '@');
-        
+
         cnt = opal_argv_count (argv);
         if (1 == cnt) {
             node_name = strdup(argv[0]);
         } else if (2 == cnt) {
-            username = strdup(argv[0]);
+            username = argv[0];
             node_name = strdup(argv[1]);
         } else {
             opal_output(0, "WARNING: Unhandled user@host-combination\n"); /* XXX */
         }
         opal_argv_free (argv);
+
+        // Strip off the FQDN if present, ignore IP addresses
+        if( !orte_keep_fqdn_hostnames && !opal_net_isaddr(node_name) ) {
+            char *ptr;
+            if (NULL != (ptr = strchr(node_name, '.'))) {
+                *ptr = '\0';
+            }
+        }
+
         /* Do we need to make a new node object? */
-        if (NULL == (node = hostfile_lookup(updates, node_name, keep_all))) {
+        if (NULL == (node = hostfile_lookup(updates, node_name))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
             node->slots = 1;
             if (NULL != username) {
-                node->username = strdup(username);
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
             }
+            opal_list_append(updates, &node->super);
         } else {
             /* add a slot */
             node->slots++;
+            free(node_name);
         }
         OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                              "%s hostfile: node %s slots %d",
@@ -306,23 +309,23 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
         /* mark the slots as "given" since we take them as being the
          * number specified via the rankfile
          */
-        node->slots_given = true;
+        ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
         /* skip to end of line */
         while (!orte_util_hostfile_done &&
                ORTE_HOSTFILE_NEWLINE != token) {
             token = orte_util_hostfile_lex();
         }
-        opal_list_append(updates, &node->super);
         return ORTE_SUCCESS;
     } else {
         hostfile_parse_error(token);
         return ORTE_ERROR;
     }
-    
+    free(username);
+
     while (!orte_util_hostfile_done) {
         token = orte_util_hostfile_lex();
-        
-        switch (token) {            
+
+        switch (token) {
         case ORTE_HOSTFILE_DONE:
             goto done;
 
@@ -330,7 +333,11 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
             goto done;
 
         case ORTE_HOSTFILE_USERNAME:
-            node->username = hostfile_parse_string();
+            username = hostfile_parse_string();
+            if (NULL != username) {
+                orte_set_attribute(&node->attributes, ORTE_NODE_USERNAME, ORTE_ATTR_LOCAL, username, OPAL_STRING);
+                free(username);
+            }
             break;
 
         case ORTE_HOSTFILE_COUNT:
@@ -341,21 +348,23 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
                 orte_show_help("help-hostfile.txt", "slots",
                                true,
                                cur_hostfile_name, rc);
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
-            if (node->slots_given) {
+            if (ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
                 /* multiple definitions were given for the
                  * slot count - this is not allowed
                  */
                 orte_show_help("help-hostfile.txt", "slots-given",
                                true,
                                cur_hostfile_name, node->name);
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
             node->slots = rc;
-            node->slots_given = true;
+            ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
 
             /* Ensure that slots_max >= slots */
             if (node->slots_max != 0 && node->slots_max < node->slots) {
@@ -369,6 +378,7 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
                 orte_show_help("help-hostfile.txt", "max_slots",
                                true,
                                cur_hostfile_name, ((size_t) rc));
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
@@ -383,29 +393,36 @@ static int hostfile_parse_line(int token, opal_list_t* updates,
                                true,
                                cur_hostfile_name, node->slots, rc);
                 ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
+                opal_list_remove_item(updates, &node->super);
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
             break;
 
+        case ORTE_HOSTFILE_STRING:
+        case ORTE_HOSTFILE_INT:
+            /* just ignore it */
+            break;
+
         default:
             hostfile_parse_error(token);
+            opal_list_remove_item(updates, &node->super);
             OBJ_RELEASE(node);
             return ORTE_ERROR;
         }
         if (number_of_slots > node->slots) {
             ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
+            opal_list_remove_item(updates, &node->super);
             OBJ_RELEASE(node);
             return ORTE_ERROR;
         }
     }
 
  done:
-    if (got_max && !node->slots_given) {
+    if (got_max && !ORTE_FLAG_TEST(node, ORTE_NODE_FLAG_SLOTS_GIVEN)) {
         node->slots = node->slots_max;
-        node->slots_given = true;
+        ORTE_FLAG_SET(node, ORTE_NODE_FLAG_SLOTS_GIVEN);
     }
-    opal_list_append(updates, &node->super);
 
     return ORTE_SUCCESS;
 }
@@ -423,7 +440,7 @@ static int hostfile_parse(const char *hostfile, opal_list_t* updates,
 
 
     cur_hostfile_name = hostfile;
-    
+
     orte_util_hostfile_done = false;
     orte_util_hostfile_in = fopen(hostfile, "r");
     if (NULL == orte_util_hostfile_in) {
@@ -518,13 +535,13 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
     if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, &adds, &exclude, false))) {
         goto cleanup;
     }
-    
+
     /* check for any relative node directives */
     for (item = opal_list_get_first(&adds);
          item != opal_list_get_end(&adds);
          item = opal_list_get_next(item)) {
         node=(orte_node_t*)item;
-        
+
         if ('+' == node->name[0]) {
             orte_show_help("help-hostfile.txt", "hostfile:relative-syntax",
                            true, node->name);
@@ -532,7 +549,7 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
             goto cleanup;
         }
     }
-    
+
     /* remove from the list of nodes those that are in the exclude list */
     while (NULL != (item = opal_list_remove_first(&exclude))) {
         nd = (orte_node_t*)item;
@@ -550,7 +567,7 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
         }
         OBJ_RELEASE(item);
     }
-    
+
     /* transfer across all unique nodes */
     while (NULL != (item = opal_list_remove_first(&adds))) {
         nd = (orte_node_t*)item;
@@ -611,7 +628,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         OBJ_DESTRUCT(&exclude);
         return rc;
     }
-    
+
     /* if the hostfile was empty, then treat it as a no-op filter */
     if (0 == opal_list_get_size(&newnodes)) {
         OBJ_DESTRUCT(&newnodes);
@@ -639,16 +656,16 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         }
         OBJ_RELEASE(item1);
     }
-    
+
     /* now check our nodes and keep or mark those that match. We can
      * destruct our hostfile list as we go since this won't be needed
      */
     OBJ_CONSTRUCT(&keep, opal_list_t);
     while (NULL != (item2 = opal_list_remove_first(&newnodes))) {
         node_from_file = (orte_node_t*)item2;
-        
+
         next = opal_list_get_next(item2);
-        
+
         /* see if this is a relative node syntax */
         if ('+' == node_from_file->name[0]) {
             /* see if we specified empty nodes */
@@ -693,7 +710,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                             opal_list_append(&keep, item1);
                         } else {
                             /* mark as included */
-                            node_from_list->mapped = true;
+                            ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
                         }
                         --num_empty;
                     }
@@ -706,7 +723,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                                    true, num_empty);
                     rc = ORTE_ERR_SILENT;
                     goto cleanup;
-                }            
+                }
             } else if ('n' == node_from_file->name[1] ||
                        'N' == node_from_file->name[1]) {
                 /* they want a specific relative node #, so
@@ -733,7 +750,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                             opal_list_append(&keep, item1);
                         } else {
                             /* mark as included */
-                            node_from_list->mapped = true;
+                            ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
                         }
                         break;
                     }
@@ -755,15 +772,11 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                  item1 != opal_list_get_end(nodes);
                  item1 = opal_list_get_next(item1)) {
                 node_from_list = (orte_node_t*)item1;
-                /* since the name in the hostfile might not match
-                 * our local name, and yet still be intended to match,
-                 * we have to check for local interfaces
-                 */
-                if (0 == strcmp(node_from_file->name, node_from_list->name) ||
-                    (0 == strcmp(node_from_file->name, "localhost") &&
-                     0 == strcmp(node_from_list->name, orte_process_info.nodename)) ||
-                    (opal_ifislocal(node_from_list->name) &&
-                     opal_ifislocal(node_from_file->name))) {
+                /* we have converted all aliases for ourself
+                 * to our own detected nodename, so no need
+                 * to check for interfaces again - a simple
+                 * strcmp will suffice */
+                if (0 == strcmp(node_from_file->name, node_from_list->name)) {
                     /* if the slot count here is less than the
                      * total slots avail on this node, set it
                      * to the specified count - this allows people
@@ -779,7 +792,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                         opal_list_append(&keep, item1);
                     } else {
                         /* mark as included */
-                        node_from_list->mapped = true;
+                        ORTE_FLAG_SET(node_from_list, ORTE_NODE_FLAG_MAPPED);
                     }
                     found = true;
                     break;
@@ -799,7 +812,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
         /* cleanup the newnode list */
         OBJ_RELEASE(item2);
     }
-    
+
     /* if we still have entries on our hostfile list, then
      * there were requested hosts that were not in our allocation.
      * This is an error - report it to the user and return an error
@@ -824,12 +837,12 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
     while (NULL != (item1 = opal_list_remove_first(nodes))) {
         OBJ_RELEASE(item1);
     }
-    
+
     /* the nodes list has been cleared - rebuild it in order */
     while (NULL != (item1 = opal_list_remove_first(&keep))) {
         opal_list_append(nodes, item1);
     }
-    
+
 cleanup:
     OBJ_DESTRUCT(&newnodes);
 
@@ -846,31 +859,31 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
     bool want_all_empty=false;
     orte_node_t *node_from_pool, *newnode;
     int rc;
-    
+
     OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s hostfile: creating ordered list of hosts from hostfile %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
-    
+
     OBJ_CONSTRUCT(&exclude, opal_list_t);
-    
+
     /* parse the hostfile and add the contents to the list, keeping duplicates */
     if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, nodes, &exclude, true))) {
         goto cleanup;
     }
-    
+
     /* parse the nodes to process any relative node directives */
     item2 = opal_list_get_first(nodes);
     while (item2 != opal_list_get_end(nodes)) {
         orte_node_t *node=(orte_node_t*)item2;
-        
+
         /* save the next location in case this one gets removed */
         item1 = opal_list_get_next(item2);
-        
+
         if ('+' != node->name[0]) {
             item2 = item1;
             continue;
         }
-        
+
         /* see if we specified empty nodes */
         if ('e' == node->name[1] ||
             'E' == node->name[1]) {
@@ -922,7 +935,7 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
                                true, num_empty);
                 rc = ORTE_ERR_SILENT;
                 goto cleanup;
-            }            
+            }
             /* since we have expanded the provided node, remove
              * it from list
              */
@@ -976,7 +989,7 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
             rc = ORTE_ERR_SILENT;
             goto cleanup;
         }
-        
+
         /* move to next */
         item2 = item1;
     }
@@ -1000,9 +1013,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
         }
         OBJ_RELEASE(item);
     }
-    
+
 cleanup:
     OBJ_DESTRUCT(&exclude);
-    
+
     return rc;
 }
