@@ -1,24 +1,25 @@
-/**
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
+/*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #ifndef MCA_MEMHEAP_H
 #define MCA_MEMHEAP_H
-#include "opal/mca/mca.h"
+#include "oshmem/mca/mca.h"
 #include "oshmem/constants.h"
 #include "oshmem/proc/proc.h"
 
 #include "oshmem/mca/sshmem/sshmem.h"
 #include "oshmem/mca/spml/spml.h"
 
-#define DEFAULT_SYMMETRIC_HEAP_SIZE      256
-#define SIZE_IN_MEGA_BYTES(size_in_mb)  size_in_mb * 1024 * 1024
 
 BEGIN_C_DECLS
 struct mca_memheap_base_module_t;
@@ -34,8 +35,7 @@ typedef struct memheap_context
 /**
  * Component initialize
  */
-typedef struct mca_memheap_base_module_t* (*mca_memheap_base_component_init_fn_t)(memheap_context_t *,
-                                                                                  int *priority);
+typedef int (*mca_memheap_base_component_init_fn_t)(memheap_context_t *);
 
 /*
  * Symmetric heap allocation. Malloc like interface
@@ -63,14 +63,6 @@ typedef uint64_t (*mca_memheap_base_module_find_offset_fn_t)(int pe,
                                                              void* va,
                                                              void* rva);
 
-/**
- * @return mkey suitable to access pe via given transport id. rva is set to virtual address mapping of (va)
- * on remote pe. 
- */
-typedef sshmem_mkey_t * (*mca_memheap_base_module_get_cached_mkey_fn_t)(int pe,
-                                                                          void* va,
-                                                                          int transport_id,
-                                                                          void** rva);
 typedef sshmem_mkey_t * (*mca_memheap_base_module_get_local_mkey_fn_t)(void* va,
                                                                          int transport_id);
 
@@ -84,9 +76,9 @@ typedef int (*mca_memheap_base_is_memheap_addr_fn_t)(const void* va);
 /* get mkeys from all ranks */
 typedef void (*mca_memheap_base_mkey_exchange_fn_t)(void);
 
-/* 
- * memheap component descriptor. Contains component version, information and 
- * init functions 
+/*
+ * memheap component descriptor. Contains component version, information and
+ * init functions
  */
 struct mca_memheap_base_component_2_0_0_t {
     mca_base_component_t memheap_version; /**< version */
@@ -107,7 +99,7 @@ struct mca_memheap_base_module_t {
     mca_memheap_base_module_realloc_fn_t            memheap_realloc;
     mca_memheap_base_module_free_fn_t               memheap_free;
 
-    /* 
+    /*
      * alloc/free that should be used for internal allocation.
      * Internal memory does not count towards
      *  symmetric heap memory
@@ -115,7 +107,6 @@ struct mca_memheap_base_module_t {
     mca_memheap_base_module_alloc_fn_t              memheap_private_alloc;
     mca_memheap_base_module_free_fn_t               memheap_private_free;
 
-    mca_memheap_base_module_get_cached_mkey_fn_t    memheap_get_cached_mkey;
     mca_memheap_base_module_get_local_mkey_fn_t     memheap_get_local_mkey;
     mca_memheap_base_module_find_offset_fn_t        memheap_find_offset;
     mca_memheap_base_is_memheap_addr_fn_t           memheap_is_symmetric_addr;
@@ -133,8 +124,7 @@ typedef struct mca_memheap_base_module_t mca_memheap_base_module_t;
  * Macro for use in components that are of type rcache
  */
 #define MCA_MEMHEAP_BASE_VERSION_2_0_0 \
-    MCA_BASE_VERSION_2_0_0, \
-"memheap", 2, 0, 0
+    OSHMEM_MCA_BASE_VERSION_2_1_0("memheap", 2, 0, 0)
 
 /*
  * macro for doing direct call / call through struct
