@@ -1,12 +1,15 @@
 /**
- *   Copyright (c) 2013      Mellanox Technologies, Inc.
- *                           All rights reserved.
- *     $COPYRIGHT$
+ * Copyright (c) 2013      Mellanox Technologies, Inc.
+ *                         All rights reserved.
+ * Copyright (c) 2014-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
+ * $COPYRIGHT$
  *
- *       Additional copyrights may follow
+ * Additional copyrights may follow
  *
- *         $HEADER$
- *          */
+ * $HEADER$
+ */
+
 #include "oshmem_config.h"
 #include "scoll_fca.h"
 #include <stdio.h>
@@ -83,7 +86,7 @@ static int _internal_barrier(mca_scoll_fca_module_t *fca_module)
     return rc;
 #else
     long pSync = _SHMEM_SYNC_VALUE;
-    /*we use 4th algorithm for barrier from scoll/basic.  It does not use pSync, 
+    /*we use 4th algorithm for barrier from scoll/basic.  It does not use pSync,
      * so we pass to that function just regular long value in order to meet function defenition requirements*/
     return fca_module->previous_barrier(fca_module->comm,
             &pSync,
@@ -101,7 +104,7 @@ static int have_remote_peers(struct oshmem_group_t *group,
                              size_t size,
                              int *local_peers)
 {
-    struct oshmem_proc_t *proc;
+    struct ompi_proc_t *proc;
     size_t i;
     int ret;
 
@@ -109,7 +112,7 @@ static int have_remote_peers(struct oshmem_group_t *group,
     ret = 0;
     for (i = 0; i < size; ++i) {
         proc = group->proc_array[i];
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
             ++*local_peers;
         } else {
             ret = 1;
@@ -125,15 +128,15 @@ static int have_remote_peers(struct oshmem_group_t *group,
 static int _get_local_ranks(mca_scoll_fca_module_t *fca_module)
 {
     struct oshmem_group_t *comm = fca_module->comm;
-    oshmem_proc_t* proc;
+    ompi_proc_t* proc;
     int i, rank;
 
     /* Count the local ranks */
     fca_module->num_local_procs = 0;
     for (rank = 0; rank < comm->proc_count; ++rank) {
         proc = comm->proc_array[rank];
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
-            if (proc->proc_name.vpid == (uint32_t) fca_module->rank) {
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
+            if (proc->super.proc_name.vpid == (uint32_t) fca_module->rank) {
                 fca_module->local_proc_idx = fca_module->num_local_procs;
             }
             ++fca_module->num_local_procs;
@@ -151,7 +154,7 @@ static int _get_local_ranks(mca_scoll_fca_module_t *fca_module)
     i = 0;
     for (rank = 0; rank < comm->proc_count; ++rank) {
         proc = comm->proc_array[rank];
-        if (FCA_IS_LOCAL_PROCESS(proc->proc_flags)) {
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
             fca_module->local_ranks[i++] = rank;
         }
     }
@@ -228,7 +231,7 @@ static int _fca_comm_new(mca_scoll_fca_module_t *fca_module)
     if (root_pe == comm->my_pe) {
         for (i = 0; i < comm->proc_count; i++) {
             if (mca_scoll_fca_component.rcounts[i] > 0) {
-                MCA_SPML_CALL(get((void *)mca_scoll_fca_component.my_info_exchangeable, mca_scoll_fca_component.rcounts[i], (void*)(((char*)all_info)+disps[i]),comm->proc_array[i]->proc_name.vpid));
+                MCA_SPML_CALL(get((void *)mca_scoll_fca_component.my_info_exchangeable, mca_scoll_fca_component.rcounts[i], (void*)(((char*)all_info)+disps[i]),comm->proc_array[i]->super.proc_name.vpid));
             }
         }
     }
@@ -424,7 +427,7 @@ static int mca_scoll_fca_module_enable(mca_scoll_base_module_t *module,
 
     exit_fatal:
     /* it is possible that other pe(s) succesfully enabled fca.
-     * So differnt frameworks will be used for collective ops 
+     * So differnt frameworks will be used for collective ops
      */
     FCA_ERROR("FCA module enable failed - aborting to prevent inconsistent application state");
     oshmem_shmem_abort(-1);
@@ -544,7 +547,7 @@ mca_scoll_fca_comm_query(struct oshmem_group_t *comm, int *priority)
 
     exit_fatal:
     /* it is possible that other pe(s) succesfully initialized fca.
-     * So differnt frameworks will be used for collective ops 
+     * So differnt frameworks will be used for collective ops
      */
     FCA_ERROR("FCA module query failed - aborting");
     oshmem_shmem_abort(-1);
