@@ -1,11 +1,13 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2014      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /**
@@ -43,8 +45,8 @@ struct oshmem_lock_prev_pe_container {
 };
 typedef struct oshmem_lock_prev_pe_container oshmem_lock_prev_pe_container_t;
 
-oshmem_lock_counter_t *lock_counter_head;
-oshmem_lock_prev_pe_container_t *lock_prev_pe_container_head;
+oshmem_lock_counter_t *lock_counter_head = NULL;
+oshmem_lock_prev_pe_container_t *lock_prev_pe_container_head = NULL;
 static int *lock_turn;
 static int *lock_inform;
 static int *lock_last_ticket;
@@ -72,7 +74,7 @@ static int shmem_lock_pack_pe_next(void *lock,
 static void shmem_lock_increment_counter(void *lock, int lock_size);
 static int shmem_lock_decrement_counter(void *lock, int lock_size);
 
-static int shmem_lock_get_server(const void *lock);
+static int shmem_lock_get_server(void *lock);
 static int shmem_lock_is_mine(void *lock, int lock_size);
 
 static int shmem_lock_get_ticket(void *lock);
@@ -166,9 +168,10 @@ int shmem_lock_finalize()
     return OSHMEM_SUCCESS;
 }
 
-static int shmem_lock_get_server(const void *lock)
+static int shmem_lock_get_server(void *lock)
 {
-    return (int) (((uintptr_t) lock) / 8) % shmem_n_pes();
+    uint64_t offset =  MCA_MEMHEAP_CALL(find_offset(shmem_my_pe(), 0, lock, lock));
+    return (offset / 8) % shmem_n_pes();
 }
 
 static uint64_t get_lock_value(const void *lock, int lock_size)
