@@ -3,9 +3,9 @@
  *                         All rights reserved.
  * Copyright (c) 2013 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -19,12 +19,18 @@
 #include "oshmem/op/op.h"
 #include "stdio.h"
 
+#if OSHMEM_PROFILING
+#include "oshmem/shmem/fortran/profile/pbindings.h"
+SHMEM_GENERATE_WEAK_BINDINGS(SHMEM_INT8_ADD, shmem_int8_add)
+#include "oshmem/shmem/fortran/profile/defines.h"
+#endif
+
 SHMEM_GENERATE_FORTRAN_BINDINGS_SUB (void,
         SHMEM_INT8_ADD,
         shmem_int8_add_,
         shmem_int8_add__,
         shmem_int8_add_f,
-        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T value, MPI_Fint *pe), 
+        (FORTRAN_POINTER_T target, FORTRAN_POINTER_T value, MPI_Fint *pe),
         (target,value,pe) )
 
 void shmem_int8_add_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T value, MPI_Fint *pe)
@@ -32,11 +38,11 @@ void shmem_int8_add_f(FORTRAN_POINTER_T target, FORTRAN_POINTER_T value, MPI_Fin
     ompi_fortran_integer8_t out_value = 0;
     oshmem_op_t* op = oshmem_op_sum_fint8;
 
-    MCA_ATOMIC_CALL(fadd(FPTR_2_VOID_PTR(target), 
-        (void *)&out_value, 
-        FPTR_2_VOID_PTR(value), 
-        sizeof(out_value), 
-        OMPI_FINT_2_INT(*pe), 
+    MCA_ATOMIC_CALL(fadd(FPTR_2_VOID_PTR(target),
+        (void *)&out_value,
+        FPTR_2_VOID_PTR(value),
+        sizeof(out_value),
+        OMPI_FINT_2_INT(*pe),
         op));
 }
 
